/*
 ************************************************************************
 *                                                                      *
 * template_avl.c -- c version of template_avl.py                       *
 *                                                                      *
 *            Written by  Nitin Bhagat @ UDRI                           *
 *            Modified by John Dannenhoffer @ Syracuse University       *
 *                                                                      *
 ************************************************************************
 */

/*
 * This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *     MA  02110-1301  USA
 */

#include "caps.h"
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>

#ifdef WIN32
   #define unlink      _unlink
   #define getcwd      _getcwd
   #define snprintf    _snprintf
   #define PATH_MAX    _MAX_PATH
   #define strcasecmp  stricmp
#else
   #include <limits.h>
   #include <unistd.h>
#endif

#define STANDALONE 1


/***********************************************************************/
/*                                                                     */
/*   helper functions for dealing with getting/setting values          */
/*                                                                     */
/***********************************************************************/

static void
printErrors(int nErr, capsErrs *errors)
{
  int         i, j, stat, eType, nLines;
  char        **lines;
  capsObj     obj;
  static char *type[5] = {"Cont:   ", "Info:   ", "Warning:", "Error:  ",
                          "Status: "};

  if (errors == NULL) return;

  for (i = 1; i <= nErr; i++) {
    stat = caps_errorInfo(errors, i, &obj, &eType, &nLines, &lines);
    if (stat != CAPS_SUCCESS) {
      printf(" printErrors: %d/%d caps_errorInfo = %d\n", i, nErr, stat);
      continue;
    }
    for (j = 0; j < nLines; j++) {
      if (j == 0) {
        printf(" CAPS %s ", type[eType+1]);
      } else {
        printf("               ");
      }
      printf("%s\n", lines[j]);
    }
  }
  
  caps_freeError(errors);
}


static int
setValueD(capsObj probObj,              /* (in)  problem object */
          int     type,                 /* (in)  variable type */
          char    name[],               /* (in)  variable name */
          double  value,                /* (in)  value to set */
          const char *units)            /* (in)  units of value */
{
    int  status = CAPS_SUCCESS;

    capsObj    valObj;
    int        *partial=NULL;

    int       nErr;
    capsErrs  *errors;

    /* --------------------------------------------------------------- */

    status = caps_childByName(probObj, VALUE, type, name,
                              &valObj, &nErr, &errors);
    if (nErr != 0) printErrors(nErr, errors);
    if (status != CAPS_SUCCESS) {
        printf("caps_childByName(%s) -> status=%d\n", name, status);
        return status;
    }

    status = caps_setValue(valObj, Double, 1, 1, (void *)(&value), partial,
                           units, &nErr, &errors);
    if (nErr != 0) printErrors(nErr, errors);
    if (status != CAPS_SUCCESS)
        printf("caps_setValue(%s) -> status=%d\n", name, status);

    return status;
}


static int
getValueD(capsObj   analObj,            /* (in)  analysis object */
          int       type,               /* (in)  variable type */
          char      name[],             /* (in)  variable name */
          int       *nErr,              /* (out) number of errors */
          capsErrs  **errors,           /* (out) array  of errors */
          double    *value)             /* (out) value */
{
    int  status = CAPS_SUCCESS;

    capsObj          valObj;
    const double     *data;
    int              nrow, ncol;
    const int        *partial;
    const char       *units;
    enum capsvType   vtype;

    /* --------------------------------------------------------------- */

    *value = 0.0;

    status = caps_childByName(analObj, VALUE, type, name,
                              &valObj, nErr, errors);
    if (*nErr != 0) printErrors(*nErr, *errors);
    if (status != CAPS_SUCCESS) return status;

    status = caps_getValue(valObj, &vtype, &nrow, &ncol, (const void **)(&data),
                           &partial, &units, nErr, errors);
    if (status != CAPS_SUCCESS) {
        printf("caps_getValue -> status=%d\n", status);
    } else if ((vtype != Double && vtype != DoubleDeriv) || nrow*ncol != 1) {
        printf("caps_getValue(%s) was a expecting a single Double\n", name);
        status = CAPS_BADTYPE;
    } else {
      *value = data[0];
    }

    return status;
}

/***********************************************************************/
/*                                                                     */
/*   run_avl - run AVL with given .csm file                            */
/*                                                                     */
/***********************************************************************/

int
run_avl(char   filename[],              /* (in)  name of .csm or .cdc file */
        double coefs[])                 /* (out) force/mement coefficients */
                                        /*       [0] CLtot  lift    coefficient */
                                        /*       [1] CDtot  drag    coefficient */
                                        /*       [2] CXtot  X-force coefficient */
                                        /*       [3] CYtot  Y-force coefficient */
                                        /*       [4] CZtot  Z-force coefficient */
                                        /*       [5] Cltot  yaw     coefficient */
                                        /*       [6] Cmtot  pitch   coefficient */
                                        /*       [7] Cntot  roll    coefficient */
                                        /*       [8] e      Oswald efficiency */
{
    int        status = 0;              /* (out) return status */

    int i; // Indexing

    // CAPS objects
    capsObj          problemObj, avlObj, valObj;
    capsErrs         *errors;

    // CAPS return values
    int              nErr, nrowWing=0, ncolWing=0, nrowHtail=0, ncolHtail=0;
    int              ihinge, jhinge, nsurface=0, nhinge=0, exec;
    double           *wHinge, *htHinge, temp;
    char             *stringVal=NULL, tempStr[1024];
    const int        *partial;
    const char       *units;
    enum capsvType   vtype;
    capsTuple        *surfaceTuple=NULL, *hingeTuple=NULL;

    /* --------------------------------------------------------------- */

    /* Add ".csm" if filename does not end with eiter .csm or .cdc */
    if (strcmp(&(filename[strlen(filename)-4]), ".csm") != 0 &&
        strcmp(&(filename[strlen(filename)-4]), ".cdc") != 0   ) {
        strcat(filename, ".csm");
    }

    /* Initialize capsProblem object from [.csm] file */
    printf("\n==> Loading geometry from file \"%s\"...", filename);
    status = caps_open("design_avl", NULL, 0, filename, 1, &problemObj,
                       &nErr, &errors);
    if (nErr != 0) printErrors(nErr, errors);
    if (status != CAPS_SUCCESS) {
        printf("caps_open -> status=%d\n", status);
        goto cleanup;
    }

    /* Set geometry variables to enable Avl mode */
    printf("\n==> Setting Build Variables and Geometry Values...\n");

    status = setValueD(problemObj, GEOMETRYIN, "VIEW:Concept", 0, NULL);
    if (status != CAPS_SUCCESS) {
        printf("setValueD(VIEW:Concept) -> status=%d\n", status);
        goto cleanup;
    }

    status = setValueD(problemObj, GEOMETRYIN, "VIEW:Vlm", 1, NULL);
    if (status != CAPS_SUCCESS) {
        printf("setValueD(VIEW:VLM) -> status=%d\n", status);
        goto cleanup;
    }

    /* Set other geometry specific variables */
//$$$    status = setValueD(myGeometry, GEOMETRYIN, "wing:aspect", 12, NULL);
//$$$    if (status != CAPS_SUCCESS) {printf("setValueD(wong:aspect) -> status=%d\n", status); goto cleanup;}

    /* Create AVL AIM (auto execution is disabled) */
    printf("\n==> Creating AVL aim...\n");
    exec   = 0;
    status = caps_makeAnalysis(problemObj, "avlAIM", NULL,
          "{\"mass\":\"kg\", \"length\":\"m\", \"time\":\"s\", \"temperature\":\"K\"}",
                               NULL, &exec, &avlObj, &nErr, &errors);
    if (nErr != 0) printErrors(nErr, errors);
    if (status != CAPS_SUCCESS) {
        printf("caps_makeAnalysis -> status=%d\n", status);
        goto cleanup;
    }

    /* Set analysis specific variables */
    status = setValueD(avlObj, ANALYSISIN, "Mach", 0.5, NULL);
    if (status != CAPS_SUCCESS) {
        printf("setValueD(Mach) -> status=%d\n", status);
        goto cleanup;
    }

    status = setValueD(avlObj, ANALYSISIN, "Alpha", 10.0, "degree");
    if (status != CAPS_SUCCESS) {
        printf("setValueD(Alpha) -> status=%d\n", status);
        goto cleanup;
    }

    status = setValueD(avlObj, ANALYSISIN, "Beta", 0.0, "degree");
    if (status != CAPS_SUCCESS) {
        printf("setValueD(Beta) -> status=%d\n", status);
        goto cleanup;
    }

    /* Tuple that will accumulate AVL_Surface info */
    surfaceTuple = (capsTuple *) EG_alloc(2*sizeof(capsTuple));  /* enough room for Wing and Htail */
    if (surfaceTuple == NULL) {
        status = EGADS_MALLOC;
        goto cleanup;
    }

    /* If .csm file has a Wing, set up AVL-specific variables for the wing */
    status = getValueD(problemObj, GEOMETRYIN, "COMP:wing", &nErr, &errors,
                       &temp);
    if (nErr != 0) printErrors(nErr, errors);
    if (status == CAPS_SUCCESS) {
        surfaceTuple[nsurface].name  = EG_strdup("Wing");
        surfaceTuple[nsurface].value = EG_strdup("{\"numChord\": 10, \"spaceChord\": 1.0, \"numSpanTotal\": 60, \"spaceSpan\": 1.0}");
        nsurface++;
    }

    /* If .csm file has a Htail, set up AVL-specific variables for the htail */
    status = getValueD(problemObj, GEOMETRYIN, "COMP:htail", &nErr, &errors,
                       &temp);
    if (nErr != 0) printErrors(nErr, errors);
    if (status == CAPS_SUCCESS) {
        surfaceTuple[nsurface].name  = EG_strdup("Htail");
        surfaceTuple[nsurface].value = EG_strdup("{\"numChord\": 10, \"spaceChord\": 1.0, \"numSpanTotal\": 40, \"spaceSpan\": 1.0}");
        nsurface++;
    }

    /* Tell AVL about the surfaces */
    status = caps_childByName(avlObj, VALUE, ANALYSISIN, "AVL_Surface",
                              &valObj, &nErr, &errors);
    if (nErr != 0) printErrors(nErr, errors);
    if (status != CAPS_SUCCESS) {
        printf("caps_childByName -> status=%d\n", status);
        goto cleanup;
    }

    status = caps_setValue(valObj, Tuple, nsurface, 1, (void **) surfaceTuple,
                           NULL, NULL, &nErr, &errors);
    if (nErr != 0) printErrors(nErr, errors);
    if (status != CAPS_SUCCESS) {
        printf("caps_setValue(AVL_Surface) -> status=%d\n", status);
        goto cleanup;
    }

    /* Set up control surface deflections */
    status = caps_childByName(problemObj, VALUE, GEOMETRYIN, "wing:hinge",
                              &valObj, &nErr, &errors);
    if (nErr != 0) printErrors(nErr, errors);
    if (status != CAPS_SUCCESS) {
        printf("caps_childByName(wing:hinge) -> status=%d\n", status);
    } else {
        status = caps_getValue(valObj, &vtype, &nrowWing, &ncolWing,
                               (const void **)(&wHinge), &partial, &units,
                               &nErr, &errors);
        if (nErr != 0) printErrors(nErr, errors);
        if (status != CAPS_SUCCESS) {
            printf("caps_getValue(wing:hinge) -> status=%d\n", status);
            goto cleanup;
        }

        if (vtype != Double || nrowWing*ncolWing%9 != 0) {
            printf("vlen must be multiple of 9 Doubles\n");
            status = CAPS_BADTYPE;
            goto cleanup;
        }

        nhinge += nrowWing*ncolWing / 9;
    }

    status = caps_childByName(problemObj, VALUE, GEOMETRYIN, "htail:hinge",
                              &valObj, &nErr, &errors);
    if (nErr != 0) printErrors(nErr, errors);
    if (status != CAPS_SUCCESS) {
      printf("caps_childByName(htail:hinge) -> status=%d\n", status);
    } else {
        status = caps_getValue(valObj, &vtype, &nrowHtail, &ncolHtail,
                               (const void**)(&htHinge), &partial, &units,
                               &nErr, &errors);
        if (nErr != 0) printErrors(nErr, errors);
        if (status != CAPS_SUCCESS) {
            printf("caps_getValue(hting:hinge) -> status=%d\n", status);
            goto cleanup;
        }

        if (vtype != Double || nrowHtail*ncolHtail%9 != 0) {
            printf("vlen must be multiple of 9 Doubles\n");
            status = CAPS_BADTYPE;
            goto cleanup;
        }

        nhinge += nrowHtail*ncolHtail / 9;
    }

    if (nhinge > 0) {
        ihinge = 0;

        status = caps_childByName(avlObj, VALUE, ANALYSISIN, "AVL_Control",
                                  &valObj, &nErr, &errors);
        if (nErr != 0) printErrors(nErr, errors);
        if (status != CAPS_SUCCESS) {
            printf("caps_childByName(AVL_Control) -> status=%d\n", status);
            goto cleanup;
        }

        hingeTuple = (capsTuple *) EG_alloc(nhinge*sizeof(capsTuple));
        if (hingeTuple == NULL) {
            status = EGADS_MALLOC;
            goto cleanup;
        }

        /* make entry for each wing control surface */
        for (jhinge = 0; jhinge < nrowWing*ncolWing/9; jhinge++) {
            snprintf(tempStr, 1023, "wingHinge%d", jhinge+1);
            hingeTuple[ihinge].name  = EG_strdup(tempStr);
            snprintf(tempStr, 1023, "{\"deflectionAngle\": %f}", wHinge[9*jhinge]);
            hingeTuple[ihinge].value = EG_strdup(tempStr);
            ihinge++;
        }

        for (jhinge = 0; jhinge < nrowHtail*ncolHtail/9; jhinge++) {
            snprintf(tempStr, 1023, "htailHinge%d", jhinge+1);
            hingeTuple[ihinge].name  = EG_strdup(tempStr);
            snprintf(tempStr, 1023, "{\"deflectionAngle\": %f}", htHinge[9*jhinge]);
            hingeTuple[ihinge].value = EG_strdup(tempStr);
            ihinge++;
        }

        status = caps_setValue(valObj, Tuple, nhinge, 1, (void **)hingeTuple,
                               NULL, NULL, &nErr, &errors);
        if (nErr != 0) printErrors(nErr, errors);
        if (status != CAPS_SUCCESS) {
            printf("caps_setValue(AVL_Control) -> status=%d\n", status);
            goto cleanup;
        }
    }

    /* Rum AIM pre-analysis */
    printf("\n==> Running AVL pre-analysis...\n");
    status = caps_preAnalysis(avlObj, &nErr, &errors);
    if (nErr != 0) printErrors(nErr, errors);
    if (status != CAPS_SUCCESS) {
        printf("caps_preAnalysis -> status=%d\n", status);
        goto cleanup;
    }

    /* Run AVL via system call */
    printf("\n ==> Running AVL...\n");
#ifdef WIN32
    caps_system(avlObj, NULL, "avl.exe caps < avlInput.txt > avlOutput.txt");
#else
    caps_system(avlObj, NULL, "avl     caps < avlInput.txt > avlOutput.txt");
#endif

    /* Run AIM post-analysis */
    printf("\n==> Running AVL post-analysis...\n");
    status = caps_postAnalysis(avlObj, &nErr, &errors);
    if (nErr != 0) printErrors(nErr, errors);
    if (status != CAPS_SUCCESS) {
        printf("caps_postAnalysis -> status=%d\n", status);
        goto cleanup;
    }

    /* Build array of force/moment coefficients */
    status = getValueD(avlObj, ANALYSISOUT, "CLtot", &nErr, &errors, &(coefs[0]));
    if (status != CAPS_SUCCESS) {
        printf("getValueD(CLtot) -> status=%d\n", status);
        goto cleanup;
    }

    status = getValueD(avlObj, ANALYSISOUT, "CDtot", &nErr, &errors, &(coefs[1]));
    if (status != CAPS_SUCCESS) {
        printf("getValueD(CDtot) -> status=%d\n", status);
        goto cleanup;
    }

    status = getValueD(avlObj, ANALYSISOUT, "CXtot", &nErr, &errors, &(coefs[2]));
    if (status != CAPS_SUCCESS) {
        printf("getValueD(CXtot) -> status=%d\n", status);
        goto cleanup;
    }

    status = getValueD(avlObj, ANALYSISOUT, "CYtot", &nErr, &errors, &(coefs[3]));
    if (status != CAPS_SUCCESS) {
        printf("getValueD(CYtot) -> status=%d\n", status);
        goto cleanup;
    }

    status = getValueD(avlObj, ANALYSISOUT, "CZtot", &nErr, &errors, &(coefs[4]));
    if (status != CAPS_SUCCESS) {
        printf("getValueD(CZtot) -> status=%d\n", status);
        goto cleanup;
    }

    status = getValueD(avlObj, ANALYSISOUT, "Cltot", &nErr, &errors, &(coefs[5]));
    if (status != CAPS_SUCCESS) {
        printf("getValueD(Cltot) -> status=%d\n", status);
        goto cleanup;
    }

    status = getValueD(avlObj, ANALYSISOUT, "Cmtot", &nErr, &errors, &(coefs[6]));
    if (status != CAPS_SUCCESS) {
        printf("getValueD(Cmtot) -> status=%d\n", status);
        goto cleanup;
    }

    status = getValueD(avlObj, ANALYSISOUT, "Cntot", &nErr, &errors, &(coefs[7]));
    if (status != CAPS_SUCCESS) {
        printf("getValueD(Cntot) -> status=%d\n", status);
        goto cleanup;
    }

    status = getValueD(avlObj, ANALYSISOUT, "e",     &nErr, &errors, &(coefs[8]));
    if (status != CAPS_SUCCESS)
        printf("getValueD(e) -> status=%d\n", status);

cleanup:
    if (status != CAPS_SUCCESS) printf("\n\nPremature exit - status = %d\n",
                                       status);

    if (surfaceTuple != NULL) {
        for (i = 0; i < nsurface; i++) {
            if (surfaceTuple[i].name  != NULL) EG_free(surfaceTuple[i].name);
            if (surfaceTuple[i].value != NULL) EG_free(surfaceTuple[i].value);
        }
        EG_free(surfaceTuple);
    }

    if (hingeTuple != NULL) {
        for (i = 0; i < nhinge; i++) {
            if (hingeTuple[i].name  != NULL) EG_free(hingeTuple[i].name);
            if (hingeTuple[i].value != NULL) EG_free(hingeTuple[i].value);
        }
        EG_free(hingeTuple);
    }

    if (stringVal != NULL) EG_free(stringVal);

    i = 0;
    if (status == CAPS_SUCCESS) i = 1;
    (void) caps_close(problemObj, i, NULL);

    return status;
}


/***********************************************************************/
/*                                                                     */
/*   main - main program                                               */
/*                                                                     */
/***********************************************************************/

#ifdef STANDALONE
int
main(int       argc,                    /* (in)  number of arguments */
     char      *argv[])                 /* (in)  array  of arguments */
{
    int        status = 0;              /* (out) return status */

    double     coefs[9];
    char       filename[PATH_MAX];

    /* --------------------------------------------------------------- */

    /* Get the name of the input file (default to ../ESP/transport.csm) */
    if (argc < 2) {
#ifdef WIN32
        strncpy(filename, "..\\ESP\\transport.csm", PATH_MAX-1);
#else
        strncpy(filename, "../ESP/transport.csm",   PATH_MAX-1);
#endif
    } else {
        strncpy(filename, argv[1],                  PATH_MAX-1);
    }

    status = run_avl(filename, coefs);
    if (status != CAPS_SUCCESS) {
        printf("run_avl -> status=%d\n", status);
        goto cleanup;
    }

    printf("\nForce/moment coefficients:\n");
    printf("    CLtot = %f\n", coefs[0]);
    printf("    CDtot = %f\n", coefs[1]);
    printf("    CXtot = %f\n", coefs[2]);
    printf("    CYtot = %f\n", coefs[3]);
    printf("    CZtot = %f\n", coefs[4]);
    printf("    Cltot = %f\n", coefs[5]);
    printf("    Cmtot = %f\n", coefs[6]);
    printf("    Cntot = %f\n", coefs[7]);
    printf("    e     = %f\n", coefs[8]);

cleanup:
    return status;
}
#endif
