#------------------------------------------------------------------------------#
# Copyright (C) 2018  John F. Dannenhoffer, III (Syracuse University)
#
# This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#     MA  02110-1301  USA
#------------------------------------------------------------------------------#

# Allow print statement to be compatible between different versions of Python
from __future__ import print_function

# Import os
import os

#------------------------------------------------------------------------------#

# Function to run AVL (called at bottom of file if top-level)
def run_avl(filename):
    """
    run AVL to compute force coefficients

    inputs:
        filename    name of .csm or .cpc file
    outputs:
        coefs       dictionary containing force/moment coefficients:
                    CLtot  lift coefficient
                    CDtot  drag coefficient
                    CXtot  X-force coefficient
                    CYtot  Y-force coefficient
                    CZtot  Z-force coefficient
                    Cltot  rolling  moment coefficient
                    Cmtot  pitching moment coefficient
                    Cntot  yawing   moment coefficient
                    e      Oswald efficiency factor
    """

    # Allow Python to interact with OS
    import os
    import sys

    # Import pyCAPS
    import pyCAPS

    # Add ".csm" if filename does not end with either .csm or .cdc
    if filename[-4:] != ".csm" and filename[-4:] != ".cpc":
        filename += ".csm"

    # Define units
    m    = pyCAPS.Unit("meter")
    kg   = pyCAPS.Unit("kg")
    s    = pyCAPS.Unit("s")
    K    = pyCAPS.Unit("Kelvin")
    deg  = pyCAPS.Unit("degree")

    # Set working directory
    problemName = "design_avl"

    # Load geometry [.csm] file
    print ("\n==> Loading geometry from file \""+filename+"\"...")
    myProblem = pyCAPS.Problem(problemName, capsFile=filename)

    # Set geometry variables to enable Avl mode
    print ("\n==> Setting Build Variables and Geometry Values...")
    try:
        myProblem.geometry.cfgpmtr.VIEW.Concept = 0
        myProblem.geometry.cfgpmtr.VIEW.Vlm     = 1
    except:
        print ("File \""+filename+"\" not set up to run Avl")
        raise SystemError

    # Set other geometry specific variables
#    myGeometry.despmtr.wing.aspect = 12.0

    # Create AVL AIM
    print ("\n==> Creating AVL aim...")
    avl = myProblem.analysis.create(aim  = "avlAIM",
                                    name = "avl",
                                    unitSystem={"mass":kg, "length":m, "time":s, "temperature":K},
                                    autoExec = False)

    # Set analysis specific variables
    avl.input.Mach  = 0.5
    avl.input.Alpha = 10.0*deg
    avl.input.Beta  = 0.0*deg

    # If .csm file has a Wing, set up AVL-specific variables for the wing
    if myProblem.geometry.cfgpmtr.COMP.wing:
        haveWing = True
        wing = {"numChord"     : 10,   # number of chordwise vorticies
                "spaceChord"   : 1.0,  # chordwise vortex spacing parameter (cosine)
                "numSpanTotal" : 60,   # number of spanwise  vorticies
                "spaceSpan"    : 1.0}  # spanwise  vortex spacing parameter (cosine)
    else:
        haveWing = False

    # If .csm file has a Htail, set up AVL-specific variables for the htail
    if myProblem.geometry.cfgpmtr.COMP.htail:
        haveHtail = True
        htail = {"numChord"     : 10,   # number of chordwise vorticies
                 "spaceChord"   : 1.0,  # chordwise vortex spacing parameter (cosine)
                 "numSpanTotal" : 40,   # number of spanwise  vorticies
                 "spaceSpan"    : 1.0}  # spanwise  vortex spacing parameter (cosine)
    else:
        haveHtail = False

    # Tell AVL about the surfaces
    if haveWing and haveHtail:
        avl.input.AVL_Surface = {"Wing": wing, "Htail":htail}
    elif haveWing:
        avl.input.AVL_Surface = {"Wing": wing}

    # Set up control surface deflections
    outHinge = {}

    try:
        hinge = myProblem.geometry.despmtr.wing.hinge
        for i in range(len(hinge)):
            outHinge["wingHinge"+str(i+1)] = {"deflectionAngle": hinge[i][0]}
    except AttributeError:
        pass

    try:
        hinge = myProblem.geometry.despmtr.htail.hinge
        for i in range(len(hinge)):
            outHinge["htailHinge"+str(i+1)] = {"deflectionAngle": hinge[i][0]}
    except AttributeError:
        pass

    if len(outHinge) > 0:
        print(outHinge)
        avl.input.AVL_Control = outHinge

    # Run AIM
    print ("\n==> Running AVL...")
    avl.runAnalysis()

    # Build dictionary of force/moment coefficients
    coefs = {}

    # Forces/moments in body axis
    coefs["CXtot"] = avl.output.CXtot
    coefs["CYtot"] = avl.output.CYtot
    coefs["CZtot"] = avl.output.CZtot
    coefs["Cltot"] = avl.output.Cltot
    coefs["Cmtot"] = avl.output.Cmtot
    coefs["Cntot"] = avl.output.Cntot

    # Forces/moments in stability axis
    coefs["CLtot"] = avl.output.CLtot
    coefs["CDtot"] = avl.output.CDtot

    # Oswald efficiency factor
    coefs["e"    ] = avl.output.e

    print("\n================ run_avl.py complete ==================\n")

    # Return the force/moment coefficients
    return coefs

#------------------------------------------------------------------------------#

# run this function if run as a script or with "python -m", but not when imported
if __name__ == "__main__":

    # Allow Python to interact with OS
    import sys

    # Get the name of the input file (default to wing1.csm)
    if len(sys.argv) < 2:
        filename = os.path.join("..","ESP","transport.csm")
    else:
        filename = sys.argv[1]

    coefs = run_avl(filename)

    print ("\nForce/moment coefficients:")
    for name, value in coefs.items():
        print (name.ljust(7) + ": " + repr(value))
