#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import OpenMDAO v3 module
import openmdao
import openmdao.api as om

#------------------------------------------------------------------------------#

class msesAnalysis(om.ExplicitComponent):
    
    def initialize(self):
        # Setup arguments for the component.
        
        # CAPS Problem input
        self.options.declare('capsProblem', types=object)

    def setup(self):
        # Assign initial values to the variables.
 
        capsProblem = self.options['capsProblem']
        mses = capsProblem.analysis['mses']

        # attach parameters to OpenMDAO object
        self.add_input('Alpha', val=mses.input.Alpha)
        
        # Add output metric
        self.add_output('CD')
        
        # Declare and attach partials to self
        self.declare_partials('CD','Alpha')

    def compute(self, inputs, outputs):
        # Compute functionals

        capsProblem = self.options['capsProblem']
        mses = capsProblem.analysis['mses']

        print("--> Alpha", inputs['Alpha'])

        # Update input values if changed
        if mses.input.Alpha != inputs['Alpha']:
            mses.input.Alpha = inputs['Alpha']

        # Grab objective and attach as an output
        outputs['CD'] = mses.output.CD
        print("--> CD", outputs['CD'])

    def compute_partials(self,inputs,partials):
        # Compute functional partial derivatives
        
        capsProblem = self.options['capsProblem']
        mses = capsProblem.analysis['mses']

        print("--> Alpha", inputs['Alpha'])

        # Update input values if changed
        if mses.input.Alpha != inputs['Alpha']:
            mses.input.Alpha = inputs['Alpha']

        # Get derivatives and set partials
        partials['CD', 'Alpha'] = mses.output["CD"].deriv("Alpha")
        print("--> CD_alpha", partials['CD', 'Alpha'])


#------------------------------------------------------------------------------#
# Setup CAPS Problem
#------------------------------------------------------------------------------#

# Initiate CAPS problem
capsProblem = pyCAPS.Problem(problemName="design_mses",
                             capsFile="naca.csm",
                             outLevel=0)

# Setup MSES
mses = capsProblem.analysis.create(aim = "msesAIM", name="mses")

# Set flow condition
mses.input.Alpha = 3.0   # Initial guess away from solution Alpha == 0
mses.input.Mach  = 0.5
mses.input.Re = 5e6

# Set meshing parameters
mses.input.GridAlpha = 0
mses.input.Airfoil_Points = 201

# Trip the flow near the leading edge to get smooth gradient
mses.input.xTransition_Upper = 0.1
mses.input.xTransition_Lower = 0.1

#------------------------------------------------------------------------------#
# Setup OpenMDAO Problem
#------------------------------------------------------------------------------#

# Setup the openmdao problem object    
omProblem = om.Problem()

# Create the OpenMDAO component
msesSystem = msesAnalysis(capsProblem = capsProblem)

# add subsystem to model
omProblem.model.add_subsystem('msesSystem', msesSystem)

# add design variables to model
omProblem.model.add_design_var('msesSystem.Alpha', lower=-5, upper=5)

# add objective to minimize CD
omProblem.model.add_objective('msesSystem.CD')

# setup the optimization
omProblem.driver = om.ScipyOptimizeDriver()
omProblem.driver.options['optimizer'] = "L-BFGS-B"
omProblem.driver.options['tol'] = 1.e-9
omProblem.driver.options['disp'] = True

# Press go
print ("\n==> Starting Optimization...")
omProblem.setup()
omProblem.run_driver()
omProblem.cleanup()

print("--> Optimized value:", omProblem.get_val("msesSystem.Alpha"))
