#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import numpy and matplotlib
import numpy as npy
import matplotlib.pyplot as plt

#------------------------------------------------------------------------------#

# Load geometry [.csm] file
filename = "naca.csm"
print ("\n==> Loading geometry from file \""+filename+"\"...")
capsProblem = pyCAPS.Problem(problemName = "design_mses",
                             capsFile = filename,
                             outLevel = 0)

# Setup MSES
mses = capsProblem.analysis.create(aim = "msesAIM", name="mses")

# Set flow condition
mses.input.Alpha = 0.0
mses.input.Mach  = 0.5
mses.input.Re = 5e6

# Set meshing parameters
mses.input.GridAlpha = 0
mses.input.Airfoil_Points = 201

# Trip the flow near the leading edge to get smooth gradient
mses.input.xTransition_Upper = 0.1
mses.input.xTransition_Lower = 0.1

# Use camber as the design variable
mses.input.Design_Variable = {"camber":{}}

# Plot the functional and gradient
cambers = npy.linspace(-0.1, 0.1, 15)
CL2 = []
CL2_camber = []
for camber in cambers:
    print("--> camber", camber)
    capsProblem.geometry.despmtr.camber = camber

    CL        = mses.output["CL"].value
    CL_camber = mses.output["CL"].deriv("camber")
    
    CL2.append( CL**2 )
    CL2_camber.append( 2*CL*CL_camber ) # Chain rule

# Compute central difference derivatives
dCL2_camber = []
for i in range(1,len(cambers)-1):
    dCL2_camber.append( (CL2[i+1]-CL2[i-1])/(cambers[i+1]-cambers[i-1]) )

fig, ax1 = plt.subplots()
ax2 = ax1.twinx()

# Plot the functional
lns1 = ax1.plot(cambers, CL2, 'o--', label = r"$C_L^2(\mathrm{camber})$", color='blue')

# Plot plot the derivative
lns2 = ax2.plot(cambers, CL2_camber, 's-', label = r"$\partial C_L^2/\partial \mathrm{camber}$", color='red')

# Plot plot the FD derivative
lns3 = ax2.plot(cambers[1:-1], dCL2_camber, '.--', label = r"$\Delta C_L^2/\Delta \mathrm{camber}$", color='black')

ax1.axvline(0., linewidth=1, linestyle=":", color='k')
ax2.axhline(0., linewidth=1, linestyle=":", color='r')

ax1.set_xlabel(r"camber")
ax1.set_ylabel(r"$C_L^2$", color='blue')
ax2.set_ylabel(r"$\partial C_L^2$", color='red')

# add legend
lns = lns1+lns2+lns3
labs = [l.get_label() for l in lns]
ax1.legend(lns, labs, loc='upper center', facecolor='white', framealpha=1)

fig.tight_layout()  # otherwise the right y-label is slightly clipped
plt.show()
