#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import OpenMDAO v3 module
import openmdao
import openmdao.api as om

#------------------------------------------------------------------------------#

class msesAnalysis(om.ExplicitComponent):
    
    def initialize(self):
        # Setup arguments for the component.
        
        # CAPS Problem input
        self.options.declare('capsProblem', types=object)

    def setup(self):
        # Assign initial values to the variables.
 
        capsProblem = self.options['capsProblem']
        naca = capsProblem.geometry

        # attach parameters to OpenMDAO object
        self.add_input('camber', val=naca.despmtr.camber)
        
        # Add output metric
        self.add_output('CL2')

        # Declare and attach partials to self
        self.declare_partials('CL2','camber')

    def compute(self, inputs, outputs):
        # Compute functionals

        capsProblem = self.options['capsProblem']
        naca = capsProblem.geometry
        mses = capsProblem.analysis['mses']

        print("--> camber", inputs['camber'])

        # Update input values if changed
        if naca.despmtr.camber != inputs['camber']:
            naca.despmtr.camber = inputs['camber']

        # Grab objective and attach as an output
        outputs['CL2']  = mses.output.CL**2
        print("--> CL2   ", outputs['CL2'])

    def compute_partials(self,inputs,partials):
        # Compute functional partial derivatives
        
        capsProblem = self.options['capsProblem']
        naca = capsProblem.geometry
        mses = capsProblem.analysis['mses']

        print("--> camber    ", inputs['camber'])

        # Update input values if changed
        if naca.despmtr.camber != inputs['camber']:
            naca.despmtr.camber = inputs['camber']

        CL        = mses.output["CL"].value
        CL_camber = mses.output["CL"].deriv("camber")

        # Get derivatives and set partials
        partials['CL2', 'camber'] = 2*CL*CL_camber
        print("--> CL2_camber", partials['CL2', 'camber'])


#------------------------------------------------------------------------------#
# Setup CAPS Problem
#------------------------------------------------------------------------------#

# Initiate CAPS problem
capsProblem = pyCAPS.Problem(problemName="design_mses",
                             capsFile="naca.csm",
                             outLevel=0)

# Set a non-zero camber. The optimization should drive it back to zero
capsProblem.geometry.despmtr.camber = 0.1

# Setup MSES
mses = capsProblem.analysis.create(aim = "msesAIM", name="mses")

# Set flow condition
mses.input.Alpha = 0.0
mses.input.Mach  = 0.5
mses.input.Re = 5e6

# Set meshing parameters
mses.input.GridAlpha = 0
mses.input.Airfoil_Points = 201

# Trip the flow near the leading edge to get smooth gradient
mses.input.xTransition_Upper = 0.1
mses.input.xTransition_Lower = 0.1

# Compute derivatives w.r.t. camber
mses.input.Design_Variable = {"camber":{}}

#------------------------------------------------------------------------------#
# Setup OpenMDAO Problem
#------------------------------------------------------------------------------#

# Setup the openmdao problem object    
omProblem = om.Problem()

# Create the OpenMDAO component
msesSystem = msesAnalysis(capsProblem = capsProblem)

# add subsystem to model
omProblem.model.add_subsystem('msesSystem', msesSystem)

# add design variables to model
omProblem.model.add_design_var('msesSystem.camber', lower=-0.1, upper=0.1)

# add objective to minimize CD
omProblem.model.add_objective('msesSystem.CL2')

# setup the optimization
omProblem.driver = om.ScipyOptimizeDriver()
omProblem.driver.options['optimizer'] = "L-BFGS-B"
omProblem.driver.options['tol'] = 1.e-8
omProblem.driver.options['disp'] = True

# Press go
print ("\n==> Starting Optimization...")
omProblem.setup()
omProblem.run_driver()
omProblem.cleanup()

print("--> Optimized value:", omProblem.get_val("msesSystem.camber"))
