#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import OpenMDAO v3 module
import openmdao
import openmdao.api as om

#------------------------------------------------------------------------------#

class masstranAnalysis(om.ExplicitComponent):
    
    def initialize(self):
        # Setup arguments for the component.
        
        # CAPS Problem input
        self.options.declare('capsProblem', types=object)

    def setup(self):
        # Assign initial values to the variables.
 
        capsProblem = self.options['capsProblem']
        geometry = capsProblem.geometry
        masstran = capsProblem.analysis['masstran']

        # attach parameters to OpenMDAO object
        self.add_input('x1', val=masstran.input.Design_Variable["x1"]["initialValue"])
        self.add_input('x2', val=masstran.input.Design_Variable["x2"]["initialValue"])
        
        # Add output metric
        self.add_output('dCGx')

        # Declare and attach partials to self
        self.declare_partials('dCGx','x1')
        self.declare_partials('dCGx','x2')

    def compute(self, inputs, outputs):
        # Compute functionals

        capsProblem = self.options['capsProblem']
        geometry = capsProblem.geometry
        masstran = capsProblem.analysis['masstran']

        print("--> x1", inputs['x1'])
        print("--> x2", inputs['x2'])

        # Update input values
        if masstran.input.Design_Variable["x1"]["initialValue"] != inputs['x1'] or \
           masstran.input.Design_Variable["x2"]["initialValue"] != inputs['x2']:
            masstran.input.Design_Variable = {
                "x1": {"initialValue":float(inputs['x1'])},
                "x2": {"initialValue":float(inputs['x2'])},
            }

        x_cg_d  = geometry.despmtr.xcg
        x_cg    = masstran.output.CG[0]

        # Grab objective and attach as an output
        outputs['dCGx'] = (x_cg - x_cg_d)**2
        print("--> dCGx", outputs['dCGx'])

    def compute_partials(self,inputs,partials):
        # Compute functional partial derivatives

        capsProblem = self.options['capsProblem']
        geometry = capsProblem.geometry
        masstran = capsProblem.analysis['masstran']

        print("--> x1", inputs['x1'])
        print("--> x2", inputs['x2'])

        # Update input values
        if masstran.input.Design_Variable["x1"]["initialValue"] != inputs['x1'] or \
           masstran.input.Design_Variable["x2"]["initialValue"] != inputs['x2']:
            masstran.input.Design_Variable = {
                "x1": {"initialValue":float(inputs['x1'])},
                "x2": {"initialValue":float(inputs['x2'])},
            }

        x_cg_d  = geometry.despmtr.xcg
        x_cg    = masstran.output.CG[0]

        cg_x1 = masstran.output["CG"].deriv("x1")[0]
        cg_x2 = masstran.output["CG"].deriv("x2")[0]

        # Get derivatives and set partials
        partials['dCGx', 'x1'] = 2*(x_cg - x_cg_d)*cg_x1
        partials['dCGx', 'x2'] = 2*(x_cg - x_cg_d)*cg_x2
        print("--> dCGx_x1", partials['dCGx', 'x1'])
        print("--> dCGx_x2", partials['dCGx', 'x2'])


#------------------------------------------------------------------------------#
# Setup CAPS Problem
#------------------------------------------------------------------------------#

capsProblem = pyCAPS.Problem("design_f118", capsFile="f118-D.csm", outLevel=0)
masstran = capsProblem.analysis.create(aim = "masstranAIM", name="masstran")

# Set mesh generation inputs
masstran.input.Edge_Point_Min = 20
masstran.input.Edge_Point_Max = 20
masstran.input.Quad_Mesh = True

# Set materials
madeupium    = {"materialType" : "isotropic",
                "density"      : 10}
unobtainium  = {"materialType" : "isotropic",
                "density"      : 25}
masstran.input.Material = {"madeupium": madeupium, "unobtainium": unobtainium}

# Set properties
shellTail = {"propertyType"      : "Shell",
             "membraneThickness" : 0.5,
             "material"          : "unobtainium"}
shellFuse = {"propertyType"      : "Shell",
             "membraneThickness" : 0.2,
             "material"          : "madeupium"}
shellWing = {"propertyType"      : "Shell",
             "membraneThickness" : 0.1,
             "material"          : "unobtainium"}
masstran.input.Property = {"wing" : shellWing, "htail": shellTail, 
                           "vtail": shellTail, "fuse" : shellFuse}

# Specify design variables
masstran.input.Design_Variable = {
    "x1": {"initialValue":shellWing["membraneThickness"]},
    "x2": {"initialValue":0},
}

# Set design variable relations
relationWing = {"componentType": "Property",
                "componentName": "wing",
                "fieldName" : "membraneThickness",
                "constantCoeff" : 0.0,
                "variableName" : ["x1","x2"],
                "linearCoeff" : [1.0, 0.1]}
relationFuse = {"componentType": "Property",
                "componentName": "fuse",
                "fieldName" : "membraneThickness",
                "constantCoeff" : 0,
                "variableName" : "x1",
                "linearCoeff" : 1.0}
masstran.input.Design_Variable_Relation = {"wingRel":relationWing,"fuseRel":relationFuse}

#------------------------------------------------------------------------------#
# Setup OpenMDAO Problem
#------------------------------------------------------------------------------#

# Setup the openmdao problem object    
omProblem = om.Problem()

# Create the OpenMDAO component
masstranSystem = masstranAnalysis(capsProblem = capsProblem)

# add subsystem to model
omProblem.model.add_subsystem('masstranSystem', masstranSystem)

# add design variables to model
omProblem.model.add_design_var('masstranSystem.x1', lower=0.01, upper=10)
omProblem.model.add_design_var('masstranSystem.x2', lower=0.01, upper=10)

# add objective to minimize CD
omProblem.model.add_objective('masstranSystem.dCGx')

# setup the optimization
omProblem.driver = om.ScipyOptimizeDriver()
omProblem.driver.options['optimizer'] = "L-BFGS-B"
omProblem.driver.options['tol'] = 1.e-9
omProblem.driver.options['disp'] = True

# Press go
print ("\n==> Starting Optimization...")
omProblem.setup()
omProblem.run_driver()
omProblem.cleanup()

print("==> Optimized value:")
print("--> x1", omProblem.get_val("masstranSystem.x1"))
print("--> x2", omProblem.get_val("masstranSystem.x2"))
