#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os
import os

#------------------------------------------------------------------------------#

# Load geometry [.csm] file
filename=os.path.join("..","ESP","transport.csm")
print ("\n==> Loading geometry from file \""+filename+"\"...")
capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = filename,
                             outLevel = 0)

# Alias the geometry
transport = capsProblem.geometry

# Set geometry variables to enable Vortex Lattice bodies
print ("\n==> Setting Build Variables and Geometry Values...")

# Change to VLM view
transport.cfgpmtr.VIEW.Concept = 0
transport.cfgpmtr.VIEW.Vlm     = 1

# Enable lifting surfaces and controls
transport.cfgpmtr.COMP.wing     = 1
transport.cfgpmtr.COMP.fuse     = 0
transport.cfgpmtr.COMP.htail    = 1
transport.cfgpmtr.COMP.vtail    = 1
transport.cfgpmtr.COMP.controls = 1


# Create AVL AIM
print ("\n==> Creating AVL aim...")
avl = capsProblem.analysis.create(aim = "avlAIM",
                                  name = "avl")

# Check the geometry
#avl.geometry.view()

# Set analysis specific variables
avl.input.Mach  =  0.5
avl.input.Alpha = 10.0
avl.input.Beta  =  0.0

# Set meshing parameters for each surface
wing = {"numChord"     : 4,
        "numSpanTotal" : 80}

htail = {"numChord"     : 4,
         "numSpanTotal" : 30}

vtail = {"numChord"     : 4,
         "numSpanTotal" : 10}

# Associate the surface parameters with capsGroups defined on the geometry
avl.input.AVL_Surface = {"Wing" : wing ,
                         "Htail": htail,
                         "Vtail": vtail}

# Set up control surface deflections based on the information in the csm file
controls = {}

hinge = transport.despmtr.wing.hinge
for i in range(len(hinge)):
    controls["WingControl_"+str(int(hinge[i][8]))] = {"deflectionAngle": hinge[i][0]}

hinge = transport.despmtr.htail.hinge
for i in range(len(hinge)):
    controls["HtailControl_"+str(int(hinge[i][8]))] = {"deflectionAngle": hinge[i][0]}

hinge = transport.despmtr.vtail.hinge
controls["VtailControl_"+str(int(hinge[8]))] = {"deflectionAngle": hinge[0]}

avl.input.AVL_Control = controls

# Explicitly trigger execution
avl.runAnalysis()

# Print CL derivatives
print ("\n==> Some stability derivatives")
print ("--> CLa  =", avl.output["CLa"].value ) # - Alpha
print ("--> CLb  =", avl.output["CLb"].value ) # - Beta
print ("--> CLp' =", avl.output["CLp'"].value) # - Roll rate
print ("--> CLq' =", avl.output["CLq'"].value) # - Pitch rate
print ("--> CLr' =", avl.output["CLr'"].value) # - Yaw rate

# Get neutral point, CG and MAC
Xnp = avl.output.Xnp
Xcg = avl.output.Xcg
MAC = transport.outpmtr.wing.mac

StaticMargin = (Xnp - Xcg)/MAC
print ("--> Xcg = ", Xcg)
print ("--> Xnp = ", Xnp)
print ("--> StaticMargin = ", StaticMargin)

ControlStability = avl.output.ControlStability
for key in ControlStability.keys():
    print("-->", key, ControlStability[key])
