#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os
import os

#------------------------------------------------------------------------------#
# Define units
m    = pyCAPS.Unit("meter")
kg   = pyCAPS.Unit("kg")
s    = pyCAPS.Unit("s")
K    = pyCAPS.Unit("Kelvin")
deg  = pyCAPS.Unit("degree")
ft   = pyCAPS.Unit("ft")
slug = pyCAPS.Unit("slug")
lb   = pyCAPS.Unit("lb")

unitSystem={"mass":kg, "length":m, "time":s, "temperature":K}

#------------------------------------------------------------------------------#

# Load geometry [.csm] file
filename=os.path.join("..","ESP","transport.csm")
print ("\n==> Loading geometry from file \""+filename+"\"...")
capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = filename,
                             outLevel = 0)

# Alias the geometry
transport = capsProblem.geometry

# Set geometry variables to enable Vortex Lattice bodies
print ("\n==> Setting Build Variables and Geometry Values...")

# Change to VLM view
transport.cfgpmtr.VIEW.Concept = 0
transport.cfgpmtr.VIEW.Vlm     = 1

# Enable lifting surfaces without controls
transport.cfgpmtr.COMP.wing     = 1
transport.cfgpmtr.COMP.fuse     = 0
transport.cfgpmtr.COMP.htail    = 1
transport.cfgpmtr.COMP.vtail    = 1
transport.cfgpmtr.COMP.controls = 0


# Create AVL AIM
print ("\n==> Create AVL aim...")
avl = capsProblem.analysis.create(aim = "avlAIM",
                                  name = "avl",
                                  unitSystem = unitSystem)

# Check the geometry
#avl.geometry.view()

# Set analysis specific variables
avl.input.Mach  = 0.5
avl.input.Alpha = 1.5*deg
avl.input.Beta  = 0.0*deg

# Set meshing parameters for each surface
wing = {"numChord"     : 4,
        "numSpanTotal" : 80}

htail = {"numChord"     : 4,
         "numSpanTotal" : 30}

vtail = {"numChord"     : 4,
         "numSpanTotal" : 10}

# Associate the surface parameters with capsGroups defined on the geometry
avl.input.AVL_Surface = {"Wing" : wing ,
                         "Htail": htail,
                         "Vtail": vtail}


# Inspired by the b737.mass avl example file
#                   mass           CGx  CGy CGz        Ixx     Iyy     Izz
cockpit   = {"mass": 3000 * lb, "CG":[  8,  0,  5] * ft, "massInertia":[0.   ,  0.   ,  0.   ] * lb*ft**2}
wing      = {"mass":19420 * lb, "CG":[ 78,  0, -1] * ft, "massInertia":[8.0e6,  0.1e6,  8.1e6] * lb*ft**2}
fuselage  = {"mass":33720 * lb, "CG":[105,  0,  2] * ft, "massInertia":[0.7e6, 18.9e6, 19.6e6] * lb*ft**2}
tailcone  = {"mass":  310 * lb, "CG":[145,  0,  0] * ft, "massInertia":[0.   ,  0.   ,  0.   ] * lb*ft**2}
Htail     = {"mass":  528 * lb, "CG":[160,  0,  2] * ft, "massInertia":[0.0e6,  0.0e6,  0.0e6] * lb*ft**2}
Vtail     = {"mass":  616 * lb, "CG":[100,  0,  8] * ft, "massInertia":[0.1e6,  0.0e6,  0.1e6] * lb*ft**2}
Main_gear = {"mass": 4500 * lb, "CG":[ 76,  0, -4] * ft, "massInertia":[0.5e6,  0.0  ,  0.5e6] * lb*ft**2}
Nose_gear = {"mass": 1250 * lb, "CG":[ 36,  0, -5] * ft, "massInertia":[0.   ,  0.   ,  0.   ] * lb*ft**2}

avl.input.MassProp = {"cockpit"  : cockpit  ,
                      "wing"     : wing     ,
                      "fuselage" : fuselage ,
                      "tailcone" : tailcone ,
                      "Htail"    : Htail    ,
                      "Vtail"    : Vtail    ,
                      "Main_gear": Main_gear,
                      "Nose_gear": Nose_gear}
avl.input.Gravity  = 32.18 * ft/s**2
avl.input.Density  = 0.002378 * slug/ft**3
avl.input.Velocity = 250.0 * m/s

# Run Analysis
print ("\n==> Running Analysis")
avl.runAnalysis()

print ("\n==> Outputs")
# Get neutral point, CG and MAC
Xnp = avl.output.Xnp
Xcg = avl.output.Xcg
MAC = transport.outpmtr.wing.mac * transport.lengthUnit()

StaticMargin = (Xnp - Xcg)/MAC
print ("--> Xcg = ", Xcg)
print ("--> Xnp = ", Xnp)
print ("--> StaticMargin = ", StaticMargin)

# Get the eigen values
# ("case #", "Array of eigen values").
# The array of eigen values is of the form = [[real0,imaginary0],[real0,imaginary0],...]

EigenValues = avl.output.EigenValues
print ("--> EigenValues ", EigenValues)

# Extract just the real eigen values
EigenValues = EigenValues["case 1"]
RealEigen = []
for i in range(len(EigenValues)):
    RealEigen.append(EigenValues[i][0])
print ()
print ("--> Real Eigen Values =", RealEigen)
