#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os
import os

#------------------------------------------------------------------------------#
# Define units
m    = pyCAPS.Unit("meter")
kg   = pyCAPS.Unit("kg")
s    = pyCAPS.Unit("s")
K    = pyCAPS.Unit("Kelvin")
deg  = pyCAPS.Unit("degree")
kft  = pyCAPS.Unit("kft")

unitSystem={"mass":kg, "length":m, "time":s, "temperature":K}

#------------------------------------------------------------------------------#

# Load geometry [.csm] file
filename = os.path.join("..","ESP","transport.csm")
print ("\n==> Loading geometry from file \""+filename+"\"...")
capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = filename,
                             outLevel = 0)

# Alias the geometry
transport = capsProblem.geometry

# Change to VLM view
transport.cfgpmtr.VIEW.Concept  = 0
transport.cfgpmtr.VIEW.Vlm      = 1

# Enable lifting surfaces without controls
transport.cfgpmtr.COMP.wing     = 1
transport.cfgpmtr.COMP.fuse     = 0
transport.cfgpmtr.COMP.htail    = 1
transport.cfgpmtr.COMP.vtail    = 1
transport.cfgpmtr.COMP.controls = 0

# Create shared analysis input parameter
mach = capsProblem.parameter.create("Mach", 0.1)

# Create friction AIM
friction = capsProblem.analysis.create(aim = "frictionAIM",
                                       name = "friction")

# Link mach number and set altitude
friction.input["Mach"].link(mach)
friction.input.Altitude = 5*kft

# Create AVL AIM
avl = capsProblem.analysis.create(aim = "avlAIM",
                                  name = "avl",
                                  unitSystem = unitSystem)

# Link inputs
avl.input["Mach"].link(mach)
avl.input["CDp"].link(friction.output["CDtotal"])

# Set analysis specific variables
avl.input.Alpha = 1.5*deg
avl.input.Beta  = 0.0*deg

# Set meshing parameters for each surface
wing = {"numChord"     : 4,
        "numSpanTotal" : 80}

htail = {"numChord"     : 4,
         "numSpanTotal" : 30}

vtail = {"numChord"     : 4,
         "numSpanTotal" : 10}

# Associate the surface parameters with capsGroups defined on the geometry
avl.input.AVL_Surface = {"Wing" : wing ,
                         "Htail": htail,
                         "Vtail": vtail}

# Compute drag for a range of Mach numbers
CDtotal = []
CDp = []
machRange = [mach.value + i*0.1 for i in range(7)]
for M in machRange:
    # Set mission parameters - AIMs will be updated automatically due to link
    mach.value = M
    
    # Track profile drag added to AVL
    CDp.append(friction.output.CDtotal)

    # Track the total drag
    CDtotal.append(avl.output.CDtot)

print()
print("--> Mach =", machRange)
print("--> CDp  =", CDp)
print("--> CD   =", CDtotal)
print()
