#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os module
import os

# Import OpenMDAO v3 module
import openmdao
import openmdao.api as om

#------------------------------------------------------------------------------#

useMPI = False

class Cart3dAnalysis(om.ExplicitComponent):

    def initialize(self):
        # Setup variables so that arguments can be provided to the component.

        # CAPS Problem input
        self.options.declare('capsProblem', types=object)

    def setup(self):
        # Assign the actual values to the variables.

        capsProblem = self.options['capsProblem']
        cart3d = capsProblem.analysis['cart3d']

        # Add free design parameters to OpenMDAO object
        self.add_input('alpha', val=cart3d.input.alpha)

        # Add outputs
        self.add_output('CL2')

        # Add partials
        self.declare_partials('CL2', 'alpha')

    def compute(self, inputs, outputs):
        # Compute functionals

        capsProblem = self.options['capsProblem']
        cart3d = capsProblem.analysis['cart3d']

        # Update input values
        if cart3d.input.alpha != inputs['alpha']:
            cart3d.input.alpha = inputs['alpha']

        # Disable design sensitivity calculations
        if cart3d.input.Design_Sensitivity != False:
            cart3d.input.Design_Sensitivity = False

        # Return the functional output (triggers autoExecution of c3d_objGrad.csh)
        outputs['CL2'] = cart3d.dynout["CL2"].value

    def compute_partials(self,inputs,partials):
        # Compute partial derivatives of the functionals

        capsProblem = self.options['capsProblem']
        cart3d = capsProblem.analysis['cart3d']

        # Update input values
        if cart3d.input.alpha != inputs['alpha']:
            cart3d.input.alpha = inputs['alpha']

        # Compute analysis and/or geometric sensitivities for the Design_Functional
        if cart3d.input.Design_Sensitivity != True:
            cart3d.input.Design_Sensitivity = True

        # Get derivatives and set partials (triggers autoExecution of c3d_objGrad.csh)
        partials['CL2', 'alpha']  = cart3d.dynout["CL2"].deriv("alpha")


#------------------------------------------------------------------------------#
# Setup CAPS Problem
#------------------------------------------------------------------------------#

# Load CSM file
filename = os.path.join("..","ESP","transport.csm")
capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = filename,
                             outLevel = 0)

# Alias the geometry
transport = capsProblem.geometry

# Change to Inviscid CFD view
transport.cfgpmtr.VIEW.Concept     = 0
transport.cfgpmtr.VIEW.CfdInviscid = 1

# Enable just wing
transport.cfgpmtr.COMP.wing        = 1
transport.cfgpmtr.COMP.fuse        = 0
transport.cfgpmtr.COMP.htail       = 0
transport.cfgpmtr.COMP.vtail       = 0
transport.cfgpmtr.COMP.nacelle     = 0
transport.cfgpmtr.COMP.pylon       = 0
transport.cfgpmtr.COMP.payload     = 0
transport.cfgpmtr.COMP.controls    = 0

# Make a symmetric Hershey-bar wing
transport.despmtr.wing.camberr     = 0
transport.despmtr.wing.camberb     = 0
transport.despmtr.wing.cambert     = 0
transport.despmtr.wing.thickr      = 0.12
transport.despmtr.wing.thickb      = 0.12
transport.despmtr.wing.thickt      = 0.12
transport.despmtr.wing.alphar      = 0
transport.despmtr.wing.alphab      = 0
transport.despmtr.wing.alphat      = 0
transport.despmtr.wing.taperi      = 1.0  # inboard  taper ratio
transport.despmtr.wing.tapero      = 1.0  # outboard taper ratio
transport.despmtr.wing.dihedral    = 0
transport.despmtr.wing.sweep       = 0

# Create Cart3D AIM
cart3d = capsProblem.analysis.create(aim = "cart3dAIM", name = "cart3d")

# Set inputs
cart3d.input.Mach          = 0.8
cart3d.input.alpha         = 1.5
cart3d.input.maxCycles     = 10
cart3d.input.nDiv          = 7
cart3d.input.maxR          = 9
cart3d.input.outer_box     = 5
cart3d.input.y_is_spanwise = True

cart3d.input.Model_X_axis = "-Xb"
cart3d.input.Model_Y_axis = "-Yb"
cart3d.input.Model_Z_axis = "-Zb"

span = transport.outpmtr.wing.span
cart3d.input.Yslices = [0, 0.5*span, span]

# Set inputs added directly to aero.csh
aerocsh = ["set it_fc = 150",
           "set it_ad = 150"]

if useMPI:
    aerocsh.append("set flowCart = mpix_flowCart")
    aerocsh.append("set mpi_prefix = 'mpiexec -n 16'")

# Setup Cl**2 as an output functional
cart3d.input.Design_Functional = {"CL2": {"function":"C_L", "power":2}}

# Declare design variables
cart3d.input.Design_Variable = {"alpha":""}

#------------------------------------------------------------------------------#
# Setup OpenMDAO Problem
#------------------------------------------------------------------------------#

# Setup the OpenMDAO problem object
omProblem = om.Problem()

# Add the fluid analysis - all the options are added as arguments
cart3dSystem = Cart3dAnalysis(capsProblem = capsProblem)

# setup the optimization
omProblem.driver = om.ScipyOptimizeDriver()
omProblem.driver.options['optimizer'] = "L-BFGS-B"
omProblem.driver.options['tol'] = 1.e-6
omProblem.driver.options['disp'] = True
omProblem.driver.options['maxiter'] = 10

# Add the Cart3D subsystem
omProblem.model.add_subsystem('cart3dSystem', cart3dSystem)

# Add design variables to model
omProblem.model.add_design_var('cart3dSystem.alpha', lower=-10.0, upper=10.0)

# Set objective function to minimize Cl**2
omProblem.model.add_objective('cart3dSystem.CL2')

# Press go
omProblem.setup()
omProblem.run_driver()
omProblem.cleanup()

# Get the optimized value
opt_alpha = omProblem.get_val("cart3dSystem.alpha")
print("Optimized alpha:", opt_alpha)
