#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os module
import os

# Prevent openmdao from importing mpi4py
# For OpenMPI, importing mpi4py causes this enviornment variable to be set:
# OMPI_MCA_ess=singleton
# which in turms prevents a system call with mpirun to function
os.environ['OPENMDAO_USE_MPI'] = "0"

# Import OpenMDAO v3 module
import openmdao
import openmdao.api as om

#------------------------------------------------------------------------------#

class Fun3dAnalysis(om.ExplicitComponent):

    def initialize(self):
    # Setup variables so that arguments can be provided to the component.

        # CAPS Problem input
        self.options.declare('capsProblem', types=object)
        self.options.declare('cores', types=int)

    def setup(self):
        # Assign the actual values to the variables.
        capsProblem = self.options['capsProblem']
        fun3d = capsProblem.analysis['fun3d']

        # attach free design parameters to OpenMDAO fluid object
        self.add_input('Alpha' , val=fun3d.input.Alpha)
        self.add_input('wing:aspect', val=capsProblem.geometry.despmtr.wing:aspect)

        # Add outputs for both objective and constraints
        self.add_output('Cd2')
        self.add_output('Cl')

        # Add partials
        self.declare_partials('Cd2','Alpha')
        self.declare_partials('Cd2','wing:aspect')

        self.declare_partials('Cl','Alpha')
        self.declare_partials('Cl','wing:aspect')

    def compute(self, inputs, outputs):
        # Compute functionals
        cores = self.options['cores']
        capsProblem = self.options['capsProblem']
        fun3d = capsProblem.analysis['fun3d']

        # Update input values
        fun3d.input.Alpha = inputs['Alpha']
        capsProblem.geometry.despmtr.wing.aspect = inputs['wing:aspect']

        # Don't compute geometric sensitivities
        fun3d.input.Design_Sensitivity = False

        # Run forward analysis
        fun3d.preAnalysis()
        fun3d.system(f"mpirun -np {cores} nodet_mpi --design_run " +
                     f"--read_surface_from_file --write_mesh {fun3d.input.Proj_Name} | tee nodet.out", 'Flow')
        fun3d.postAnalysis()

        # Grab functional and attach as an output
        outputs['Cd2'] = fun3d.dynout.Cd2
        outputs['Cl']  = fun3d.dynout.Cl
        
        # Unlink the mesh to trigger morphing
        fun3d.input["Mesh"].unlink()

    def compute_partials(self,inputs,partials):
        # The actual value of the partial derivative is assigned here.
        cores = self.options['cores']
        capsProblem = self.options['capsProblem']
        fun3d = capsProblem.analysis['fun3d']

        # Request analysis and/or geometric design sensitivities for the Design_Functional
        fun3d.input.Design_Sensitivity = True

        # Run adjoint
        fun3d.preAnalysis()
        fun3d.system(f"mpirun -np {cores} dual_mpi --getgrad --outer_loop_krylov | tee dual.out",'Adjoint')
        fun3d.postAnalysis()

        # Get derivatives and set partials
        partials['Cd2', 'Alpha'      ]  = fun3d.dynout["Cd2"].deriv("Alpha")
        partials['Cd2', 'wing:aspect'] = fun3d.dynout["Cd2"].deriv("wing:aspect")

        partials['Const', 'Alpha'      ]  = fun3d.dynout["Cl"].deriv("Alpha")
        partials['Const', 'wing:aspect'] = fun3d.dynout["Cl"].deriv("wing:aspect")

#------------------------------------------------------------------------------#
# Setup CAPS Problem
#------------------------------------------------------------------------------#

# Load CSM file
filename = os.path.join("..","ESP","transport.csm")
capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = filename,
                             outLevel = 0)

# Alias the geometry
transport = capsProblem.geometry

# Change to Inviscid CFD view
transport.cfgpmtr.VIEW.Concept     = 0
transport.cfgpmtr.VIEW.CfdInviscid = 1

# Enable just wing
transport.cfgpmtr.COMP.wing        = 1
transport.cfgpmtr.COMP.fuse        = 0
transport.cfgpmtr.COMP.htail       = 0
transport.cfgpmtr.COMP.vtail       = 0
transport.cfgpmtr.COMP.nacelle     = 0
transport.cfgpmtr.COMP.pylon       = 0
transport.cfgpmtr.COMP.payload     = 0
transport.cfgpmtr.COMP.controls    = 0

#------------------------------------------------------------------------------#

# Create aflr4 AIM
aflr4 = capsProblem.analysis.create(aim  = "aflr4AIM",
                                    name = "aflr4")

# Farfield growth factor
aflr4.input.ff_cdfr = 1.4

# Scaling factor to compute AFLR4 'ref_len' parameter
aflr4.input.Mesh_Length_Factor = 5

# Edge mesh spacing discontinuity scaled interpolant and farfield meshing BC
aflr4.input.Mesh_Sizing = {"leftWing": {"edgeWeight":1.0},
                           "riteWing": {"edgeWeight":1.0},
                           "Farfield": {"bcType":"Farfield"}}

#------------------------------------------------------------------------------#

# Create AFLR3 AIM to generate the volume mesh
aflr3 = capsProblem.analysis.create(aim  = "aflr3AIM",
                                    name = "aflr3")

# Link the aflr4 Surface_Mesh as input to aflr3
aflr3.input["Surface_Mesh"].link(aflr4.output["Surface_Mesh"])

#------------------------------------------------------------------------------#

# Create Fun3D AIM
fun3d = capsProblem.analysis.create(aim  = "fun3dAIM",
                                    name = "fun3d")

# Link the aflr3 Volume_Mesh as input to fun3d
fun3d.input["Mesh"].link(aflr3.output["Volume_Mesh"])

# Set project name. Files written to analysisDir will have this name
fun3d.input.Proj_Name = "inviscidWing"

fun3d.input.Alpha = 1.0                    # AoA
fun3d.input.Mach = 0.5                     # Mach number
fun3d.input.Equation_Type = "Compressible" # Equation type
fun3d.input.Num_Iter = 5000                # Number of iterations
fun3d.input.Restart_Read = 'off'           # Do not read restart
fun3d.input.Viscous = "inviscid"           # Inviscid calculation

# Enable Mesh_Morph to save off a reference mesh
fun3d.input.Mesh_Morph = True

# Set boundary conditions via capsGroup
inviscidBC = {"bcType" : "Inviscid"}
fun3d.input.Boundary_Condition = {"Wing"    : inviscidBC,
                                  "Farfield":"farfield"}

# Setup Cd**2 as the objective functional to minimize
# and Cl for the constraint
fun3d.input.Design_Functional = {"Cd2": {"function":"Cd", "power":2},
                                 "Cl" : {"function":"Cl"}}

# Declare design variables
fun3d.input.Design_Variable = {"Alpha":"",
                               "wing:aspect":""}

#------------------------------------------------------------------------------#
# Setup OpenMDAO Problem
#------------------------------------------------------------------------------#

# Setup the openmdao problem object
omProblem = om.Problem()

# Add the fluid analysis - all the options are added as arguments
fun3dSystem = Fun3dAnalysis(cores = 10,
                            capsProblem  = capsProblem)

omProblem.model.add_subsystem('fun3dSystem', fun3dSystem)

# setup the optimization
omProblem.driver = om.ScipyOptimizeDriver()
omProblem.driver.options['optimizer'] = 'SLSQP'
#omProblem.driver.options['optimizer'] = "L-BFGS-B"
omProblem.driver.options['tol'] = 1.e-7
omProblem.driver.options['disp'] = True
omProblem.driver.options['maxiter'] = 10

# add design variables to model
aspect_limits = transport.despmtr["wing:aspect"].limits

omProblem.model.add_design_var('fun3dSystem.Alpha' , lower=-25.0, upper=25.0)
omProblem.model.add_design_var('fun3dSystem.wing:aspect', lower=aspect_limits[0], upper=aspect_limits[1])

# minimize Cd**2 subject to Cl == 0.1
omProblem.model.add_objective('fun3dSystem.Cd2')
omProblem.model.add_constraint('fun3dSystem.Cl', equals = 0.1)

# Press go
omProblem.setup()
omProblem.run_driver()
omProblem.cleanup()

# Get the optimized value
opt_Cd2    = omProblem.get_val("fun3dSystem.Cd2")
opt_Cl     = omProblem.get_val("fun3dSystem.Cl")
opt_Alpha  = omProblem.get_val("fun3dSystem.Alpha")
opt_aspect = omProblem.get_val("fun3dSystem.wing:aspect")

print("Optimized:")
print(  "Cd2        ", opt_Cd2)
print(  "Cl         ", opt_Cl)
print(  "Alpha      ", opt_Alpha)
print(  "wing:aspect", opt_aspect)
