#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os module
import os
import shutil

#------------------------------------------------------------------------------#

# Load geometry [.csm] file
filename = os.path.join("..","ESP","transport.csm")
print ("\n==> Loading geometry from file \""+filename+"\"...")
capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = filename,
                             outLevel = 0)

# Alias the geometry
wing = capsProblem.geometry

# Set geometry variables to enable Box Structure with a Support node
wing.cfgpmtr.VIEW.Concept       = 0
wing.cfgpmtr.VIEW.SimplySupport = 1

# Enable just wing
wing.cfgpmtr.COMP.wing        = 1
wing.cfgpmtr.COMP.fuse        = 0
wing.cfgpmtr.COMP.htail       = 0
wing.cfgpmtr.COMP.vtail       = 0
wing.cfgpmtr.COMP.nacelle     = 0
wing.cfgpmtr.COMP.pylon       = 0
wing.cfgpmtr.COMP.payload     = 0
wing.cfgpmtr.COMP.controls    = 0

#------------------------------------------------------------------------------#

# Create EGADS tess aim
tess = capsProblem.analysis.create(aim = "egadsTessAIM",
                                   name = "tess")

# No Tess vertexes on edges (minimial mesh)
tess.input.Edge_Point_Max = 2

# Use regularized quads
tess.input.Mesh_Elements = "Quad"

#------------------------------------------------------------------------------#

# Create astros aim
astros = capsProblem.analysis.create(aim = "astrosAIM",
                                     name = "astros", autoExec = False)

# Link the EGADS tess Surface_Mesh to astros
astros.input["Mesh"].link(tess.output["Surface_Mesh"])

# Set analysis specific variables
astros.input.Proj_Name        = "astros_modal"
astros.input.Mesh_File_Format = "Large"

# Defined Connections
connection = {"dofDependent"   : 123456,
              "connectionType" : "RigidBody"}
astros.input.Connect = {"ribRoot": connection}

# Set supports
support = {"dofSupport": 3}
astros.input.Support = {"ribRootPoint": support}

# Set constraints
constraint = {"dofConstraint" : 12456}
astros.input.Constraint = {"ribRootPoint": constraint}

# Set analysis type
eigen = { "analysisType"         : "Modal",
          "extractionMethod"     : "SINV",
          "frequencyRange"       : [0, 10],
          "numEstEigenvalue"     : 1,
          "numDesiredEigenvalue" : 10,
          "eigenNormaliztion"    : "MASS"}
astros.input.Analysis = {"EigenAnalysis": eigen}

# Set materials
unobtainium  = {"youngModulus" : 2.2E6 ,
                "poissonRatio" : .5,
                "density"      : 7850}
madeupium    = {"materialType" : "isotropic",
                "youngModulus" : 1.2E5 ,
                "poissonRatio" : .5,
                "density"      : 7850}
astros.input.Material = {"Unobtainium": unobtainium,
                         "Madeupium"  : madeupium}

# Set properties
skinShell  = {"propertyType"        : "Shell",
              "material"            : "unobtainium",
              "bendingInertiaRatio" : 1.0,
              "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
              "membraneThickness"   : 0.05}
ribShell  = {"propertyType"        : "Shell",
             "material"            : "unobtainium",
             "bendingInertiaRatio" : 1.0,
             "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
             "membraneThickness"   : 0.1}
sparShell  = {"propertyType"        : "Shell",
              "material"            : "madeupium",
              "bendingInertiaRatio" : 1.0,
              "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
              "membraneThickness"   : 0.2}

mass    =   {"propertyType" : "ConcentratedMass",
             "mass"         : 1.0E5,
                              #I11  I12  I22    I31  I32  I33
             "massInertia"  : [0.0, 0.0, 1.0E5, 0.0, 0.0, 0.0]}

astros.input.Property = {"leftWingSkin": skinShell,
                         "riteWingSkin": skinShell,
                         "wingRib"     :  ribShell,
                         "wingSpar1"   : sparShell,
                         "wingSpar2"   : sparShell,
                         "ribSupport"  : mass     }


# Run AIM pre-analysis
print ("\n==> Running mASTROS pre-analysis...")
astros.preAnalysis()

####### Run mASTROS ####################
# Declare ASTROS install directory
astrosInstallDir = os.environ['ESP_ROOT'] + os.sep + "bin" + os.sep

# Copy files needed to run astros
files = ["ASTRO.D01", "ASTRO.IDX"]
for file in files:
    try:
        shutil.copy2(astrosInstallDir + file, astros.analysisDir + os.sep + file)
    except:
        print ('Unable to copy "' + file + '"')
        raise

# Run micro-ASTROS via system call
print ("\n==> Running mASTROS...")
astros.system("mastros.exe < " + astros.input.Proj_Name +  ".dat > " + astros.input.Proj_Name + ".out")

# Remove temporary files
for file in files:
    if os.path.isfile(astros.analysisDir + os.sep + file):
        os.remove(astros.analysisDir + os.sep + file)
#######################################

# Run AIM post-analysis
print ("\n==> Running mASTROS post-analysis...")
astros.postAnalysis()

# Get list of Eigen-frequencies
freqs = astros.output.EigenFrequency

print ("\n--> Eigen-frequencies:")
for i in range(len(freqs)):
    print ("    " + repr(i+1).ljust(2) + ": " + str(freqs[i]))
