#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os module
import os
import shutil

#------------------------------------------------------------------------------#

# Load geometry [.csm] file
filename = os.path.join("..","ESP","transport.csm")
print ("\n==> Loading geometry from file \""+filename+"\"...")
capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = filename,
                             outLevel = 0)

# Alias the geometry
wing = capsProblem.geometry

# Set geometry variables to enable Box Structure with Clambed root
wing.cfgpmtr.VIEW.Concept     = 0
wing.cfgpmtr.VIEW.Cantilever  = 1

# Enable just wing
wing.cfgpmtr.COMP.wing        = 1
wing.cfgpmtr.COMP.fuse        = 0
wing.cfgpmtr.COMP.htail       = 0
wing.cfgpmtr.COMP.vtail       = 0
wing.cfgpmtr.COMP.nacelle     = 0
wing.cfgpmtr.COMP.pylon       = 0
wing.cfgpmtr.COMP.payload     = 0
wing.cfgpmtr.COMP.controls    = 0

# reduce number of ribs for mASTROS
wing.cfgpmtr.wing.nrib1       = 1
wing.cfgpmtr.wing.nrib2       = 2
wing.cfgpmtr.wing.nrib3       = 4

#------------------------------------------------------------------------------#

# Create EGADS tess aim
tess = capsProblem.analysis.create(aim = "egadsTessAIM",
                                   name = "tess")

# No Tess vertexes on edges (minimial mesh)
tess.input.Edge_Point_Max = 2

# Use regularized quads
tess.input.Mesh_Elements = "Quad"

#------------------------------------------------------------------------------#

# Create astros aim
astros = capsProblem.analysis.create(aim = "astrosAIM",
                                     name = "astros", autoExec = False)

# Link the EGADS tess Surface_Mesh to astros
astros.input["Mesh"].link(tess.output["Surface_Mesh"])

# Set analysis specific variables
astros.input.Proj_Name        = "astros_modal"
astros.input.Mesh_File_Format = "Large"

# Set constraints
constraint = {"dofConstraint" : 123456}
astros.input.Constraint = {"rootConstraint": constraint}

# Define loads
leftLoad = {"loadType"         : "GridForce",
            "forceScaleFactor" : 1.e6,
            "directionVector"  : [0.0, 0.0, 1.0]}
riteLoad = {"loadType"         : "GridForce",
            "forceScaleFactor" : 2.e6,
            "directionVector"  : [0.0, 0.0, 1.0]}
astros.input.Load = {"leftPointLoad": leftLoad,
                     "ritePointLoad": riteLoad}

# Set analysis type
astros.input.Analysis_Type = "Static"

# Set materials
unobtainium  = {"youngModulus" : 2.2e6 ,
                "poissonRatio" : .5,
                "density"      : 7850}
madeupium    = {"materialType" : "isotropic",
                "youngModulus" : 1.2e5 ,
                "poissonRatio" : .5,
                "density"      : 7850}
astros.input.Material = {"Unobtainium": unobtainium,
                         "Madeupium"  : madeupium}

# Set properties
skinShell = {"propertyType"        : "Shell",
             "material"            : "unobtainium",
             "bendingInertiaRatio" : 1.0,
             "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
             "membraneThickness"   : 0.05}
ribShell  = {"propertyType"        : "Shell",
             "material"            : "unobtainium",
             "bendingInertiaRatio" : 1.0,
             "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
             "membraneThickness"   : 0.1}
sparShell = {"propertyType"        : "Shell",
             "material"            : "madeupium",
             "bendingInertiaRatio" : 1.0,
             "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
             "membraneThickness"   : 0.2}
astros.input.Property = {"leftWingSkin": skinShell,
                         "riteWingSkin": skinShell,
                         "wingRib"     :  ribShell,
                         "wingSpar1"   : sparShell,
                         "wingSpar2"   : sparShell}

# Run AIM pre-analysis
print ("\n==> Running mASTROS pre-analysis...")
astros.preAnalysis()

####### Run mASTROS ####################
# Declare ASTROS install directory
astrosInstallDir = os.environ['ESP_ROOT'] + os.sep + "bin" + os.sep

# Copy files needed to run astros
files = ["ASTRO.D01", "ASTRO.IDX"]
for file in files:
    try:
        shutil.copy2(astrosInstallDir + file, astros.analysisDir + os.sep + file)
    except:
        print ('Unable to copy "' + file + '"')
        raise

# Run micro-ASTROS via system call
print ("\n==> Running mASTROS...")
astros.system("mastros.exe < " + astros.input.Proj_Name +  ".dat > " + astros.input.Proj_Name + ".out")

# Remove temporary files
for file in files:
    if os.path.isfile(astros.analysisDir + os.sep + file):
        os.remove(astros.analysisDir + os.sep + file)
#######################################

# Run AIM post-analysis
print ("\n==> Running mASTROS post-analysis...")
astros.postAnalysis()

print ("\n--> Maximum displacements:")
print ("--> Tmax" , astros.output.Tmax )
print ("--> T1max", astros.output.T1max)
print ("--> T2max", astros.output.T2max)
print ("--> T3max", astros.output.T3max)
