#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os module
import os

# Import TACS
from tacs import functions, pyTACS

#------------------------------------------------------------------------------#

# Load geometry [.csm] file
filename = os.path.join("..","ESP","transport.csm")
print ("\n==> Loading geometry from file \""+filename+"\"...")
capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = filename,
                             outLevel = 0)

# Alias the geometry
wing = capsProblem.geometry

# Set geometry variables to enable Box Structure with Clambed root
wing.cfgpmtr.VIEW.Concept     = 0
wing.cfgpmtr.VIEW.Cantilever  = 1

# Enable just wing
wing.cfgpmtr.COMP.wing        = 1
wing.cfgpmtr.COMP.fuse        = 0
wing.cfgpmtr.COMP.htail       = 0
wing.cfgpmtr.COMP.vtail       = 0
wing.cfgpmtr.COMP.nacelle     = 0
wing.cfgpmtr.COMP.pylon       = 0
wing.cfgpmtr.COMP.payload     = 0
wing.cfgpmtr.COMP.controls    = 0

# reduce number of ribs
wing.cfgpmtr.wing.nrib1       = 1
wing.cfgpmtr.wing.nrib2       = 2
wing.cfgpmtr.wing.nrib3       = 4

#------------------------------------------------------------------------------#

# Create EGADS tess aim
tess = capsProblem.analysis.create(aim = "egadsTessAIM",
                                   name = "tess")

# No Tess vertexes on edges (minimial mesh)
tess.input.Edge_Point_Min = 2
tess.input.Edge_Point_Max = 2

# Use regularized quads
tess.input.Mesh_Elements = "Quad"

#------------------------------------------------------------------------------#

# Create tacs aim
tacs = capsProblem.analysis.create(aim = "tacsAIM", 
                                   name = "tacs")

# Link the EGADS tess Surface_Mesh to tacs
tacs.input["Mesh"].link(tess.output["Surface_Mesh"])

#increase the precision in the BDF file
tacs.input.File_Format = "Large"
tacs.input.Mesh_File_Format = "Large" 

# set the analysis type
tacs.input.Analysis_Type = "Static"

# Set material
aluminum = {"materialType"     : "Isotropic",
            "youngModulus"     : 70.0E9,
            "poissonRatio"     : 0.35,
            "density"          : 2.7E3,
            "tensionAllow"     : 20.0E7,
            "compressionAllow" : 20.0E7,
            "yieldAllow"       : 20.0E7,
            "thermalExpCoeff"  : 23.1E-6}
tacs.input.Material = {"aluminum" : aluminum}

# Set properties
skinShell = {"propertyType"        : "Shell",
             "membraneThickness"   : 0.03,
             "material"            : "aluminum",
             "bendingInertiaRatio" : 5.0, 
             "shearMembraneRatio"  : 5.0/6.0}
ribShell  = {"propertyType"        : "Shell",
             "membraneThickness"   : 0.05,
             "material"            : "aluminum",
             "bendingInertiaRatio" : 1.0, 
             "shearMembraneRatio"  : 5.0/6.0}
sparShell = {"propertyType"        : "Shell",
             "membraneThickness"   : 0.05,
             "material"            : "aluminum",
             "bendingInertiaRatio" : 1.0, 
             "shearMembraneRatio"  : 5.0/6.0}
tacs.input.Property = {"leftWingSkin": skinShell,
                       "riteWingSkin": skinShell,
                       "wingRib"     :  ribShell,
                       "wingSpar1"   : sparShell,
                       "wingSpar2"   : sparShell}

# Set constraints
constraint = {"constraintType" : "Displacement",
              "dofConstraint" : 123456}
tacs.input.Constraint = {"rootConstraint": constraint}

# Set loads
load = {"loadType" : "GridForce",
        "directionVector" : [0.0, 0.0, 1.0],
        "forceScaleFactor" : 1.0E2}
tacs.input.Load = {"leftWingSkin": load,
                   "riteWingSkin": load}

# Run AIM pre-analysis
print ("\n==> Running TACS pre-analysis...")
tacs.preAnalysis()

####### Run TACS ####################

#initialize pytacs with that data file (serial execution)
dat_file = os.path.join(tacs.analysisDir, tacs.input.Proj_Name+".dat")
fea_solver = pyTACS(dat_file)
    
# Set up TACS Assembler
fea_solver.initialize()

#read the bdf & dat file into pytacs FEAsolver
#SPs represents "StructuralProblems"
SPs = fea_solver.createTACSProbsFromBDF()

# add mass and aggregated stress constraint functions
function_names = ["mass", "ks_vmfailure"]
for caseID in SPs:
    SPs[caseID].addFunction('mass', functions.StructuralMass)
    SPs[caseID].addFunction('ks_vmfailure', functions.KSFailure, safetyFactor=1.5, ksWeight=1000.0)

# solve each structural analysis problem (in this case 1)
print ("\n==> Running TACS...")
tacs_funcs = {}
base_name = "tacs_output"
for caseID in SPs:
    SPs[caseID].solve()
    SPs[caseID].evalFunctions(tacs_funcs,evalFuncs=function_names)
    SPs[caseID].writeSolution(baseName=base_name, outputDir=tacs.analysisDir)

# Run AIM post-analysis
print ("\n==> Running TACS post-analysis...")
tacs.postAnalysis()

# print the output analysis functions and sensitivities
print("Tacs Analysis Outputs...")
for func_name in function_names:
    # find the associated tacs key (tacs key = (loadset) + (func_name))
    for tacs_key in tacs_funcs:
        if func_name in tacs_key:
            break

    func_value = tacs_funcs[tacs_key].real
    print(f"\tfunctional {func_name} = {func_value}")

