#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os module
import os

# Import TACS
from tacs import functions, pyTACS

# Import OpenMDAO v3 module
import openmdao
import openmdao.api as om

#------------------------------------------------------------------------------#

class tacsAnalysis(om.ExplicitComponent):
    def initialize(self):
        """
        Declare the capsProblem to the openMdao component
        """
        # CAPS problem input
        self.options.declare('capsProblem', types=object)

    def setup(self):
        """
        Declare the inputs and outputs of the capsProblem to openmdao
        """
        capsProblem = self.options['capsProblem']
        tacs = capsProblem.analysis['tacs']

        # generate the list of thickness variables
        self._thick_dvs = list(tacs.input.Design_Variable.keys())

        # sort the thickness DVs in the order written to the dat file
        self._thick_dvs.sort()

        # add the thickness variables as openmdao inputs, with starting uniform thickness
        for thick_dv in self._thick_dvs:
            self.add_input(thick_dv, val=tacs.input.Design_Variable[thick_dv]["initialValue"])

        # add output functions
        self._func_names = ["mass","ks_vmfailure"]
        for func_name in self._func_names:
            self.add_output(func_name)

        # add partials
        for func_name in self._func_names:
            for thick_dv in self._thick_dvs:
                self.declare_partials(func_name, thick_dv)

        # first design boolean
        self._first_design = True

        # assuming tacs is initialized we will prepare FEA analysis of fixed geometry here
        # first generate the mesh with a bdf and dat file
        tacs.preAnalysis()

        # then initialize pytacs fea_solver and then TACS assembler
        dat_file = os.path.join(tacs.analysisDir, tacs.input.Proj_Name+".dat")
        self.fea_solver = pyTACS(dat_file)
        self.fea_solver.initialize()

        # create the list of steady structural analysis problems in TACS
        # this one is to check the design variables at the start, recreated multiple times
        self.SPs = self.fea_solver.createTACSProbsFromBDF()

    def compute(self, inputs, outputs):
        """
        compute the objective functions
        """
        # obtain the aim from openmdao storage
        capsProblem = self.options['capsProblem']
        tacs = capsProblem.analysis['tacs']

        # check for updated input values
        design_change = False
        for caseID in self.SPs:
            # gets the xarray of Struct Problem
            xarray = self.SPs[caseID].x.getArray()
        
            for ithick, thick_dv in enumerate(self._thick_dvs):
                if xarray[ithick].real != inputs[thick_dv]:
                    design_change = True

        if design_change or not hasattr(self, "_functions"):
            self._run_analysis(inputs)

        # Grab objectives and attach as openmdao outputs
        for func_name in self._func_names:
            outputs[func_name] = self._functions[func_name]
            print("-->",func_name,outputs[func_name])

    def compute_partials(self, inputs, partials):
        """
        the actual value of partial derivatives assigned here
        """
        capsProblem = self.options['capsProblem']
        tacs = capsProblem.analysis['tacs']

        # check for updated input values
        design_change = False
        for caseID in self.SPs:
            # gets the xarray of Struct Problem
            xarray = self.SPs[caseID].x.getArray()
        
            for ithick, thick_dv in enumerate(self._thick_dvs):
                if xarray[ithick].real != inputs[thick_dv]:
                    design_change = True

        if design_change or not hasattr(self, "_gradients"):
            self._run_analysis(inputs)

        # Get derivatives and partials
        for func_name in self._func_names:
            for thick_dv in self._thick_dvs:
                partials[func_name, thick_dv] = self._gradients[func_name][thick_dv]
                print("-->",func_name,thick_dv,partials[func_name, thick_dv])


    def _run_analysis(self, inputs):
        """
        run forward and adjoint TACS analysis on fixed structural mesh geometry from tacsAIM
        """
        capsProblem = self.options['capsProblem']
        tacs = capsProblem.analysis['tacs']

        # recreate the set of struct problems
        self.SPs = self.fea_solver.createTACSProbsFromBDF()

        # solve each of the TACS struct problems, forward and adjoint analysis
        tacs_funcs = {}
        tacs_sens = {}
        for caseID in self.SPs:

            # add functions for the struct problems
            self.SPs[caseID].addFunction('mass', functions.StructuralMass)
            self.SPs[caseID].addFunction('ks_vmfailure', functions.KSFailure, safetyFactor=1.5, ksWeight=50.0)

            # overwrite the thickness DVs
            xarray = self.SPs[caseID].x.getArray()
            for ithick, thick_dv in enumerate(self._thick_dvs):
                xarray[ithick] = float(inputs[thick_dv])
                print("-->",thick_dv,inputs[thick_dv])

            self.SPs[caseID].solve()
            self.SPs[caseID].evalFunctions(tacs_funcs, evalFuncs=self._func_names)
            self.SPs[caseID].evalFunctionsSens(tacs_sens, evalFuncs=self._func_names)
            self.SPs[caseID].writeSolution(baseName="tacs_output", outputDir=tacs.analysisDir)
            #print(tacs_sens)

        # store functions and gradients to be read by openmdao
        self._functions = {}
        self._gradients = {}
        for func_name in self._func_names:
            
            # corresponding tacs key for single loadset (key="loadset#" + "func_name")
            for tacs_key in tacs_funcs:
                if func_name in tacs_key:
                    break

            self._functions[func_name] = tacs_funcs[tacs_key].real
            self._gradients[func_name] = {}
            
            # grab structDV derivatives, not coordinate derivatives from tacs_sens
            struct_derivs = tacs_sens[tacs_key]["struct"]
            for ithick,thick_dv in enumerate(self._thick_dvs):
                self._gradients[func_name][thick_dv] = struct_derivs[ithick].real


#---------------------------------------------------------------------#
# Setup CAPS Problem
#---------------------------------------------------------------------#


# Load geometry [.csm] file
filename = os.path.join("..","ESP","transport.csm")
print ("\n==> Loading geometry from file \""+filename+"\"...")
capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = filename,
                             outLevel = 0)

# Alias the geometry
wing = capsProblem.geometry

# Set geometry variables to enable Box Structure with Clambed root
wing.cfgpmtr.VIEW.Concept     = 0
wing.cfgpmtr.VIEW.Cantilever  = 1

# Enable just wing
wing.cfgpmtr.COMP.wing        = 1
wing.cfgpmtr.COMP.fuse        = 0
wing.cfgpmtr.COMP.htail       = 0
wing.cfgpmtr.COMP.vtail       = 0
wing.cfgpmtr.COMP.nacelle     = 0
wing.cfgpmtr.COMP.pylon       = 0
wing.cfgpmtr.COMP.payload     = 0
wing.cfgpmtr.COMP.controls    = 0

# reduce number of ribs
wing.cfgpmtr.wing.nrib1       = 1
wing.cfgpmtr.wing.nrib2       = 2
wing.cfgpmtr.wing.nrib3       = 4

#------------------------------------------------------------------------------#

# Create EGADS tess aim
tess = capsProblem.analysis.create(aim = "egadsTessAIM",
                                   name = "tess")

# Size mesh
tess.input.Edge_Point_Min = 6
tess.input.Edge_Point_Max = 6

# Use regularized quads
tess.input.Mesh_Elements = "Quad"

#------------------------------------------------------------------------------#

# Create tacs aim
tacs = capsProblem.analysis.create(aim = "tacsAIM", 
                                   name = "tacs")

# Link the EGADS tess Surface_Mesh to tacs
tacs.input["Mesh"].link(tess.output["Surface_Mesh"])

#increase the precision in the BDF file
tacs.input.File_Format = "Large"
tacs.input.Mesh_File_Format = "Large" 

# set the analysis type
tacs.input.Analysis_Type = "Static"

# Set material
aluminum = {"materialType"     : "Isotropic",
            "youngModulus"     : 70.0E9,
            "poissonRatio"     : 0.35,
            "density"          : 2.7E3,
            "tensionAllow"     : 20.0E7,
            "compressionAllow" : 20.0E7,
            "yieldAllow"       : 20.0E7,
            "thermalExpCoeff"  : 23.1E-6}
tacs.input.Material = {"aluminum" : aluminum}

# Set properties
skinShell = {"propertyType"        : "Shell",
             "membraneThickness"   : 0.003,
             "material"            : "aluminum",
             "bendingInertiaRatio" : 5.0, 
             "shearMembraneRatio"  : 5.0/6.0}
ribShell  = {"propertyType"        : "Shell",
             "membraneThickness"   : 0.05,
             "material"            : "aluminum",
             "bendingInertiaRatio" : 1.0, 
             "shearMembraneRatio"  : 5.0/6.0}
sparShell = {"propertyType"        : "Shell",
             "membraneThickness"   : 0.1,
             "material"            : "aluminum",
             "bendingInertiaRatio" : 1.0, 
             "shearMembraneRatio"  : 5.0/6.0}
tacs.input.Property = {"leftWingSkin": skinShell,
                       "riteWingSkin": skinShell,
                       "wingRib"     :  ribShell,
                       "wingSpar1"   : sparShell,
                       "wingSpar2"   : sparShell}

# make thickness variables
thickness = skinShell["membraneThickness"]
skinThick = {"initialValue" : thickness,
             "lowerBound"   : thickness*0.5,
             "upperBound"   : thickness*1.5,
             "maxDelta"     : thickness}
thickness = ribShell["membraneThickness"]
ribThick  = {"initialValue" : thickness,
             "lowerBound"   : thickness*0.5,
             "upperBound"   : thickness*1.5,
             "maxDelta"     : thickness}
thickness = sparShell["membraneThickness"]
sparThick = {"initialValue" : thickness,
             "lowerBound"   : thickness*0.5,
             "upperBound"   : thickness*1.5,
             "maxDelta"     : thickness}
tacs.input.Design_Variable = {"skinThick": skinThick,
                              "ribThick" : ribThick,
                              "sparThick": sparThick}

relLeftSkin = {"componentType": "Property",
               "componentName": "leftWingSkin",
               "fieldName"    : "T",
               "variableName" : "skinThick"}
relRiteSkin = {"componentType": "Property",
               "componentName": "riteWingSkin",
               "fieldName"    : "T",
               "variableName" : "skinThick"}
relRib      = {"componentType": "Property",
               "componentName": "wingRib",
               "fieldName"    : "T",
               "variableName" : "ribThick"}
relSpar     = {"componentType": "Property",
               "componentName": "wingSpar1",
               "fieldName"    : "T",
               "variableName" : "sparThick"}
tacs.input.Design_Variable_Relation = {"relLeftSkin": relLeftSkin,
                                       "relRiteSkin": relRiteSkin,
                                       "relRib"     : relRib,
                                       "relSpar"    : relSpar}

# Set constraints
constraint = {"constraintType" : "Displacement",
              "dofConstraint" : 123456}
tacs.input.Constraint = {"rootConstraint": constraint}

# Set loads
load = {"loadType" : "GridForce",
        "directionVector" : [0.0, 0.0, 1.0],
        "forceScaleFactor" : 1.0e1}
tacs.input.Load = {"leftWingSkin": load,
                   "riteWingSkin": load}

#--------------------------------------------------------------------------#
# Setup OpenMDAO Problem
#--------------------------------------------------------------------------#

# setup the OpenMDAO Problem object
om_problem = om.Problem()

# Create the OpenMDAO component
tacsSystem = tacsAnalysis(capsProblem=capsProblem)
om_problem.model.add_subsystem("tacsSystem", tacsSystem)

# setup the optimizer settings
om_problem.driver = om.ScipyOptimizeDriver()
om_problem.driver.options['optimizer'] = "SLSQP"
om_problem.driver.options['tol'] = 1.e-9
om_problem.driver.options['disp'] = True

# add design variables to the model
for thick_dv in tacs.input.Design_Variable.keys():
    om_problem.model.add_design_var(f"tacsSystem.{thick_dv}", lower=0.001, upper=10.0)

# add objectives & constraints to the model
om_problem.model.add_objective("tacsSystem.mass",ref=3.5e5)
om_problem.model.add_constraint("tacsSystem.ks_vmfailure", upper=0.182)

# Start the optimization
print("\n==> Starting optimization...")
om_problem.setup()
om_problem.run_driver()
om_problem.cleanup()

# report the final optimal design
print("==> Optimized value:")
for thick_dv in tacs.input.Design_Variable.keys():
    print("--> ", thick_dv, om_problem.get_val(f"tacsSystem.{thick_dv}"))
