#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os module
import os
import shutil

#------------------------------------------------------------------------------#

capsProblem = pyCAPS.Problem(problemName = "design_Transport",
                             capsFile = os.path.join("..","ESP","transport.csm"),
                             outLevel = 0)

# Create an alias for geometry
wing = capsProblem.geometry

# Enable structures and CFD views
wing.cfgpmtr.VIEW.Concept     = 0
wing.cfgpmtr.VIEW.Cantilever  = 1
wing.cfgpmtr.VIEW.CfdInviscid = 1

# Enable just wing
wing.cfgpmtr.COMP.wing        = 1
wing.cfgpmtr.COMP.fuse        = 0
wing.cfgpmtr.COMP.htail       = 0
wing.cfgpmtr.COMP.vtail       = 0
wing.cfgpmtr.COMP.nacelle     = 0
wing.cfgpmtr.COMP.pylon       = 0
wing.cfgpmtr.COMP.payload     = 0
wing.cfgpmtr.COMP.controls    = 0

# reduce number of ribs for mASTROS
wing.cfgpmtr.wing.nrib1       = 1
wing.cfgpmtr.wing.nrib2       = 2
wing.cfgpmtr.wing.nrib3       = 4

wing.view()

#------------------------------------------------------------------------------#

# Create aflr4 AIM
aflr4 = capsProblem.analysis.create(aim = "aflr4AIM",
                                    name = "aflr4")

# Farfield growth factor
aflr4.input.ff_cdfr = 1.4

# Scaling factor to compute AFLR4 'ref_len' parameter
aflr4.input.Mesh_Length_Factor = 5

# Edge mesh spacing discontinuity scaled interpolant and farfield BC
aflr4.input.Mesh_Sizing = {"leftWing": {"edgeWeight":1.0},
                           "riteWing": {"edgeWeight":1.0},
                           "Farfield": {"bcType":"Farfield"}}

#------------------------------------------------------------------------------#

# Create AFLR3 AIM
aflr3 = capsProblem.analysis.create(aim  = "aflr3AIM",
                                    name = "aflr3")

# Link the surface mesh
aflr3.input["Surface_Mesh"].link(aflr4.output["Surface_Mesh"])

#------------------------------------------------------------------------------#

# Create fun3d AIM
fun3d = capsProblem.analysis.create(aim  = "fun3dAIM",
                                    name = "fun3d")

# Link the volume mesh
fun3d.input["Mesh"].link(aflr3.output["Volume_Mesh"])

# Set inputs for fun3d
speedofSound = 340.0 # m/s
refVelocity = 100.0 # m/s
refDensity = 1.2 # kg/m^3

fun3d.input.Proj_Name       = "aeroelasticModal"
fun3d.input.Mesh_ASCII_Flag = False
fun3d.input.Mach            = refVelocity/speedofSound
fun3d.input.Equation_Type   = "compressible"
fun3d.input.Viscous         = "inviscid"
fun3d.input.Num_Iter        = 10
fun3d.input.Time_Accuracy   = "2ndorderOPT"
fun3d.input.Time_Step       = 0.001*speedofSound
fun3d.input.Num_Subiter     = 30
fun3d.input.Temporal_Error  = 0.01

fun3d.input.CFL_Schedule      = [1, 5.0]
fun3d.input.CFL_Schedule_Iter = [1, 40]
fun3d.input.Overwrite_NML     = True
fun3d.input.Restart_Read      = "off"

inviscidBC = {"bcType" : "Inviscid"}
fun3d.input.Boundary_Condition = {"Wing": inviscidBC,
                                  "Farfield": "farfield"}

#------------------------------------------------------------------------------#

# Create EGADS tess aim
tess = capsProblem.analysis.create(aim = "egadsTessAIM")

# No Tess vertexes on edges (minimial mesh)
tess.input.Edge_Point_Max = 2

# Use regularized quads
tess.input.Mesh_Elements = "Quad"

#------------------------------------------------------------------------------#

# Create ASTROS AIMs
astros = capsProblem.analysis.create(aim  = "astrosAIM",
                                     name = "astros", autoExec = False)

# Set inputs for astros
astros.input.Proj_Name = fun3d.input.Proj_Name

astros.input.Analysis_Type = "Modal"

# Set analysis type
eigen = { "analysisType"         : "Modal",
          "extractionMethod"     : "SINV",
          "frequencyRange"       : [0, 10],
          "numEstEigenvalue"     : 1,
          "numDesiredEigenvalue" : 10,
          "eigenNormaliztion"    : "MASS"}
astros.input.Analysis = {"EigenAnalysis": eigen}

# Set constraints
constraint = {"groupName"     : ["rootConstraint"],
              "dofConstraint" : 123456}
astros.input.Constraint = {"rootConstraint": constraint}

# Set materials
unobtainium  = {"youngModulus" : 2.2E6 ,
                "poissonRatio" : .5,
                "density"      : 7850}
madeupium    = {"materialType" : "isotropic",
                "youngModulus" : 1.2E5 ,
                "poissonRatio" : .5,
                "density"      : 7850}
astros.input.Material = {"Unobtainium": unobtainium,
                         "Madeupium"  : madeupium}

# Set properties
skinShell = {"propertyType"        : "Shell",
             "material"            : "unobtainium",
             "bendingInertiaRatio" : 1.0,
             "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
             "membraneThickness"   : 0.05}
ribShell  = {"propertyType"        : "Shell",
             "material"            : "unobtainium",
             "bendingInertiaRatio" : 1.0,
             "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
             "membraneThickness"   : 0.1}
sparShell = {"propertyType"        : "Shell",
             "material"            : "madeupium",
             "bendingInertiaRatio" : 1.0,
             "shearMembraneRatio"  : 0, # Turn of shear - no materialShear
             "membraneThickness"   : 0.2}
astros.input.Property = {"leftWingSkin": skinShell,
                         "riteWingSkin": skinShell,
                         "wingRib"     :  ribShell,
                         "wingSpar1"   : sparShell,
                         "wingSpar2"   : sparShell}

#------------------------------------------------------------------------------#

# Create an array of EigenVector names
numEigenVector = 3
eigenVectors = []
for i in range(numEigenVector):
    eigenVectors.append("EigenVector_" + str(i+1))

# Create the capsBounds for data transfer
boundNames = ["upperWing", "lowerWing", "leftTip", "riteTip"]
for boundName in boundNames:
    # Create the bound
    bound = capsProblem.bound.create(boundName)

    # Create the vertex sets on the bound for fun3d and astros analysis
    fun3dVset  = bound.vertexSet.create(fun3d)
    astrosVset = bound.vertexSet.create(astros)

    # Create eigenVector data sets
    for eigenVector in eigenVectors:
        fun3d_eigenVector  = fun3dVset.dataSet.create(eigenVector)
        astros_eigenVector = astrosVset.dataSet.create(eigenVector)

        # Link the data sets
        fun3d_eigenVector.link(astros_eigenVector, "Interpolate")

    # Close the bound as complete (cannot create more vertex or data sets)
    bound.close()

#------------------------------------------------------------------------------#

# Run AIM pre-analysis
print ("\n==> Running mASTROS pre-analysis...")
astros.preAnalysis()

####### Run mASTROS ####################
# Declare ASTROS install directory
astrosInstallDir = os.environ['ESP_ROOT'] + os.sep + "bin" + os.sep

# Copy files needed to run astros
files = ["ASTRO.D01", "ASTRO.IDX"]
for file in files:
    try:
        shutil.copy2(astrosInstallDir + file, astros.analysisDir + os.sep + file)
    except:
        print ('Unable to copy "' + file + '"')
        raise

# Run micro-ASTROS via system call
print ("\n==> Running mASTROS...")
astros.system("mastros.exe < " + astros.input.Proj_Name +  ".dat > " + astros.input.Proj_Name + ".out")

# Remove temporary files
for file in files:
    if os.path.isfile(astros.analysisDir + os.sep + file):
        os.remove(astros.analysisDir + os.sep + file)
#######################################

# Run AIM post-analysis
print ("\n==> Running mASTROS post-analysis...")
astros.postAnalysis()

#------------------------------------------------------------------------------#

# Visualize capsBounds
for boundName in capsProblem.bound.keys():
    # Get the bound
    bound = capsProblem.bound[boundName]

    # Get the astros vertex set on the bound
    astrosVset = bound.vertexSet[astros.name]

    # Visualize each eigenVector data sets
    for eigenVector in eigenVectors:
        astros_eigenVector = astrosVset.dataSet[eigenVector]

        # View the data sets
        astros_eigenVector.view()

#------------------------------------------------------------------------------#

# Retrieve natural frequencies from the structural analysis
naturalFreq = astros.output.EigenRadian # rads/s
mass        = astros.output.EigenGeneralMass

modalDict = {} # Build up Modal Aeroelastic dict
for j in eigenVectors:
    modeNum = int(j.strip("EigenVector_"))

    value = {"frequency" : naturalFreq[modeNum-1],
             "damping" : 0.99,
             "generalMass" : mass[modeNum-1],
             "generalVelocity" : 0.1}

    modalDict[j] = value

# Add Eigen information in fun3d
fun3d.input.Modal_Aeroelastic          = modalDict
fun3d.input.Modal_Ref_Velocity         = refVelocity
fun3d.input.Modal_Ref_Dynamic_Pressure = 0.5*refDensity*refVelocity*refVelocity
fun3d.input.Modal_Ref_Length           = 1.0

# Run the preAnalysis
print ("\n==> Running FUN3D pre-analysis...")
fun3d.preAnalysis()

####### Run fun3d ####################
print ("\n\nRunning FUN3D......")

cmdLineOpt = "--moving_grid --aeroelastic_internal --animation_freq -1"

# Run fun3d via system call
fun3d.system("mpirun -np 4 nodet_mpi " + cmdLineOpt + " > Info.out");
#######################################

# Run AIM post-analysis
print ("\n==> Running FUN3D post-analysis...")
fun3d.postAnalysis()
