#------------------------------------------------------------------------------#

# Import pyCAPS module
import pyCAPS

# Import os module
import os

# Import OpenMDAO v3 module
import openmdao
import openmdao.api as om

#------------------------------------------------------------------------------#

class masstranAnalysis(om.ExplicitComponent):
    
    def initialize(self):
        # Setup arguments for the component.
        
        # CAPS Problem input
        self.options.declare('capsProblem', types=object)

    def setup(self):
        # Assign initial values to the variables.
 
        capsProblem = self.options['capsProblem']
        geometry = capsProblem.geometry

        # attach parameters to OpenMDAO object
        self.add_input('htail:lh', val=geometry.despmtr["htail:lh"].value)
        self.add_input('wing:xroot', val=geometry.despmtr["wing:xroot"].value)
        
        # Add output metric
        self.add_output('dCGx')

        # Declare and attach partials to self
        self.declare_partials('dCGx','htail:lh')
        self.declare_partials('dCGx','wing:xroot')

    def compute(self, inputs, outputs):
        # Compute functionals

        capsProblem = self.options['capsProblem']
        geometry = capsProblem.geometry
        masstran = capsProblem.analysis['masstran']

        print("--> htail:lh  ", inputs['htail:lh'])
        print("--> wing:xroot", inputs['wing:xroot'])

        # Update input values
        if geometry.despmtr["htail:lh"].value != inputs['htail:lh']:
            geometry.despmtr["htail:lh"].value = inputs['htail:lh']

        if geometry.despmtr["wing:xroot"].value != inputs['wing:xroot']:
            geometry.despmtr["wing:xroot"].value = inputs['wing:xroot']

        x_cg_d  = geometry.despmtr.xcg
        x_cg    = masstran.output.CG[0]

        # Grab objective and attach as an output
        outputs['dCGx'] = (x_cg - x_cg_d)**2
        print("--> dCGx", outputs['dCGx'])

    def compute_partials(self,inputs,partials):
        # Compute functional partial derivatives

        capsProblem = self.options['capsProblem']
        geometry = capsProblem.geometry
        masstran = capsProblem.analysis['masstran']

        print("--> htail:lh  ", inputs['htail:lh'])
        print("--> wing:xroot", inputs['wing:xroot'])

        # Update input values
        if geometry.despmtr["htail:lh"].value != inputs['htail:lh']:
            geometry.despmtr["htail:lh"].value = inputs['htail:lh']

        if geometry.despmtr["wing:xroot"].value != inputs['wing:xroot']:
            geometry.despmtr["wing:xroot"].value = inputs['wing:xroot']

        x_cg_d  = geometry.despmtr['xcg'].value
        x_cg    = masstran.output.CG[0]

        cg_lh = masstran.output["CG"].deriv("htail:lh")[0]
        cg_xr = masstran.output["CG"].deriv("wing:xroot")[0]

        # Get derivatives and set partials
        partials['dCGx', 'htail:lh'  ] = 2*(x_cg - x_cg_d)*cg_lh
        partials['dCGx', 'wing:xroot'] = 2*(x_cg - x_cg_d)*cg_xr
        print("--> dCGx_htail:lh", partials['dCGx', 'htail:lh'])
        print("--> dCGx_wing:xroot", partials['dCGx', 'wing:xroot'])


#------------------------------------------------------------------------------#
# Setup CAPS Problem
#------------------------------------------------------------------------------#

filename=os.path.join("..","..","data","session04","f118-D.csm")
capsProblem = pyCAPS.Problem("design_f118", capsFile=filename, outLevel=0)
masstran = capsProblem.analysis.create(aim = "masstranAIM", name="masstran")

# Set mesh generation inputs
masstran.input.Edge_Point_Min = 20
masstran.input.Edge_Point_Max = 20
masstran.input.Quad_Mesh = True

# Set materials
madeupium    = {"materialType" : "isotropic",
                "density"      : 10}
unobtainium  = {"materialType" : "isotropic",
                "density"      : 25}

masstran.input.Material = {
    "madeupium": madeupium,
    "unobtainium": unobtainium
}

# Set properties
shell = {"propertyType"      : "Shell",
         "membraneThickness" : 0.1,
         "material"          : "madeupium"}
masstran.input.Property = {
    "wing" : shell,
    "htail": shell,
    "vtail": shell,
    "fuse" : shell
}

# Specify design variables
masstran.input.Design_Variable = {
    "htail:lh": {},
    "wing:xroot": {},
}


#------------------------------------------------------------------------------#
# Setup OpenMDAO Problem
#------------------------------------------------------------------------------#

# Setup the openmdao problem object    
omProblem = om.Problem()

# Create the OpenMDAO component
masstranSystem = masstranAnalysis(capsProblem = capsProblem)

# add subsystem to model
omProblem.model.add_subsystem('masstranSystem', masstranSystem)

# add design variables to model
omProblem.model.add_design_var('masstranSystem.htail:lh', lower=2, upper=15)
omProblem.model.add_design_var('masstranSystem.wing:xroot', lower=0, upper=10)

# add objective to minimize CD
omProblem.model.add_objective('masstranSystem.dCGx')

print("==> Initial values:")
print("htail:lh  ", capsProblem.geometry.despmtr["htail:lh"].value)
print("wing:xroot", capsProblem.geometry.despmtr["wing:xroot"].value)

capsProblem.geometry.view()

# setup the optimization
omProblem.driver = om.ScipyOptimizeDriver()
omProblem.driver.options['optimizer'] = "L-BFGS-B"
omProblem.driver.options['tol'] = 1.e-9
omProblem.driver.options['disp'] = True

# Press go
print ("\n==> Starting Optimization...")
omProblem.setup()
omProblem.run_driver()
omProblem.cleanup()

capsProblem.geometry.despmtr["htail:lh"].value = omProblem.get_val("masstranSystem.htail:lh")
capsProblem.geometry.despmtr["wing:xroot"].value = omProblem.get_val("masstranSystem.wing:xroot")

print("==> Optimized values:")
print("--> htail:lh  ", capsProblem.geometry.despmtr["htail:lh"].value)
print("--> wing:xroot", capsProblem.geometry.despmtr["wing:xroot"].value)

capsProblem.geometry.view()


