#------------------------------------------------------------------------------#

# Import pyCAPS
import pyCAPS

# Import os
import os

#------------------------------------------------------------------------------#
# Define units
m    = pyCAPS.Unit("meter")
kg   = pyCAPS.Unit("kg")
s    = pyCAPS.Unit("s")
K    = pyCAPS.Unit("Kelvin")
deg  = pyCAPS.Unit("degree")
ft   = pyCAPS.Unit("ft")
slug = pyCAPS.Unit("slug")
lb   = pyCAPS.Unit("lb")

unitSystem={"mass":kg, "length":m, "time":s, "temperature":K}

#------------------------------------------------------------------------------#

# Load geometry [.csm] file
filename = os.path.join("..","..","data","ESP","transport.csm")
print ("\n==> Loading geometry from file \""+filename+"\"...")
myProblem = pyCAPS.Problem(problemName = "workDir_masstran_MultiAIM",
                           capsFile = filename,
                           outLevel = 0)
transport = myProblem.geometry

# Set geometry variables to enable Vortex Lattice bodies
print ("\n==> Setting Build Variables and Geometry Values...")

# Change to OmlStructure view
transport.cfgpmtr.VIEW.Concept      = 0
transport.cfgpmtr.VIEW.OmlStructure = 1

# Enable fuselage and lifting surfaces
transport.cfgpmtr.COMP.Wing    = 1
transport.cfgpmtr.COMP.Fuse    = 1
transport.cfgpmtr.COMP.Htail   = 1
transport.cfgpmtr.COMP.Vtail   = 1
transport.cfgpmtr.COMP.Control = 0

# Load masstran AIM
print ("\n==> Loading masstran aims...")
masstranFuse = myProblem.analysis.create(aim = "masstranAIM",
                                         name = "masstranFuse",
                                         unitSystem = unitSystem,
                                         capsIntent = "fuse")
masstranWing = myProblem.analysis.create(aim = "masstranAIM",
                                         name = "masstranWing",
                                         unitSystem = unitSystem,
                                         capsIntent = "wing")
masstranTail = myProblem.analysis.create(aim = "masstranAIM",
                                         name = "masstranTail",
                                         unitSystem = unitSystem,
                                         capsIntent = "tail")

# Set materials
fuseium  = { "density" : 100 * lb/ft**3 }
wingium  = { "density" : 200 * lb/ft**3 }
tailium  = { "density" :  50 * lb/ft**3 }
masstranFuse.input.Material = {"fuseium": fuseium}
masstranWing.input.Material = {"wingium": wingium}
masstranTail.input.Material = {"tailium": tailium}

# Set property
fuseshell = {"propertyType"      : "Shell",
             "material"          : "fuseium",
             "membraneThickness" : 0.02 * ft}
wingshell = {"propertyType"      : "Shell",
             "material"          : "wingium",
             "membraneThickness" : 0.03 * ft}
tailshell = {"propertyType"      : "Shell",
             "material"          : "tailium",
             "membraneThickness" : 0.01 * ft}


masstranFuse.input.Property = {"fuseSkin": fuseshell}
masstranWing.input.Property = {"wingSkin": wingshell}
masstranTail.input.Property = {"htailSkin": tailshell,
                               "vtailSkin": tailshell}

print ("\n==> Computing mass properties...")

fuse_mass = masstranFuse.output.Mass    
fuse_CG   = masstranFuse.output.CG      
fuse_I    = masstranFuse.output.I_Vector

fuse_skin = {"mass":fuse_mass,"CG":fuse_CG,"massInertia":fuse_I}

wing_mass = masstranWing.output.Mass    
wing_CG   = masstranWing.output.CG      
wing_I    = masstranWing.output.I_Vector

wing_skin = {"mass":wing_mass,"CG":wing_CG,"massInertia":wing_I}

tail_mass = masstranTail.output.Mass    
tail_CG   = masstranTail.output.CG      
tail_I    = masstranTail.output.I_Vector

tail_skin = {"mass":tail_mass,"CG":tail_CG,"massInertia":tail_I}

print ("\n==> Computed mass properties...")
print ("--> fuse_skin = ", fuse_skin)
print ("--> wing_skin = ", wing_skin)
print ("--> tail_skin = ", tail_skin)
