#------------------------------------------------------------------------------#

# Import pyCAPS
import pyCAPS

# Import os
import os

#------------------------------------------------------------------------------#
# Define units
m    = pyCAPS.Unit("meter")
kg   = pyCAPS.Unit("kg")
s    = pyCAPS.Unit("s")
K    = pyCAPS.Unit("Kelvin")
deg  = pyCAPS.Unit("degree")
ft   = pyCAPS.Unit("ft")
slug = pyCAPS.Unit("slug")
lb   = pyCAPS.Unit("lb")

unitSystem={"mass":kg, "length":m, "time":s, "temperature":K}

#------------------------------------------------------------------------------#

# Load geometry [.csm] file
filename = os.path.join("..","..","data","ESP","transport.csm")
print ("\n==> Loading geometry from file \""+filename+"\"...")
myProblem = pyCAPS.Problem(problemName = "workDir_masstran_MultiMat",
                           capsFile = filename,
                           outLevel = 0)
transport = myProblem.geometry

# Set geometry variables to enable Vortex Lattice bodies
print ("\n==> Setting Build Variables and Geometry Values...")

# Change to OmlStructure view
transport.cfgpmtr.VIEW.Concept      = 0
transport.cfgpmtr.VIEW.OmlStructure = 1

# Enable fuselage and lifting surfaces
transport.cfgpmtr.COMP.Wing    = 1
transport.cfgpmtr.COMP.Fuse    = 1
transport.cfgpmtr.COMP.Htail   = 1
transport.cfgpmtr.COMP.Vtail   = 1
transport.cfgpmtr.COMP.Control = 0

# Load masstran AIM
print ("\n==> Loading masstran aim...")
masstran = myProblem.analysis.create(aim = "masstranAIM",
                                     unitSystem = unitSystem)

# Set materials
fuseium  = { "density" : 100 * lb/ft**3 }
wingium  = { "density" : 200 * lb/ft**3 }
tailium  = { "density" :  50 * lb/ft**3 }
masstran.input.Material = {"fuseium": fuseium,
                           "wingium": wingium,
                           "tailium": tailium}

# Set property
fuseshell = {"propertyType"      : "Shell",
             "material"          : "fuseium",
             "membraneThickness" : 0.02 * ft}
wingshell = {"propertyType"      : "Shell",
             "material"          : "wingium",
             "membraneThickness" : 0.03 * ft}
tailshell = {"propertyType"      : "Shell",
             "material"          : "tailium",
             "membraneThickness" : 0.01 * ft}


masstran.input.Property = {"fuseSkin" : fuseshell,
                           "wingSkin" : wingshell,
                           "htailSkin": tailshell,
                           "vtailSkin": tailshell}

print ("\n==> Computing mass properties...")
aircraft_mass = masstran.output.Mass    
aircraft_CG   = masstran.output.CG      
aircraft_I    = masstran.output.I_Vector

aircraft_skin = {"mass":aircraft_mass.convert("lb"),"CG":aircraft_CG.convert("ft"),"massInertia":aircraft_I.convert("lb*ft^2")}

print ("\n==> Computed mass properties...")
print ("--> aircraft_skin = ", aircraft_skin)
